;*****************************************
;*****************************************
;***     GameBoy Sound Xperimenter     ***
;***            Version 1.3            ***
;***                                   ***
;***   Written by :                    ***
;***     MJCulross      08/29/99 2130  ***
;***                                   ***
;***   Features :                      ***
;***     Control contents of each      ***
;***     sound control register        ***
;***                                   ***
;*****************************************
;*****************************************

timer_div               equ     40
blinktime               equ     20
last_cursor             equ     $5B
last_line               equ     $24
display_col             equ     $9800

pos_0                   equ     display_col+14+(32*4)
pos_1                   equ     display_col+14+(32*5)
pos_2                   equ     display_col+14+(32*6)
pos_3                   equ     display_col+14+(32*7)
pos_4                   equ     display_col+14+(32*8)
pos_5                   equ     display_col+14+(32*9)
pos_6                   equ     display_col+14+(32*10)

pos_arrows              equ     display_col+03+(32*16)
pos_text                equ     display_col+01+(32*12)

        INCLUDE "hardware.inc"
        INCLUDE "ibmfont.inc"

        INCLUDE "memory1.asm"

; Vblank Interrupt
        SECTION "Org $40",HOME[$40]
        push    af
        push    bc
        push    de
        push    hl
        call    service_vblank_int
        pop     hl
        pop     de
        pop     bc
        pop     af
        reti

; Timer Overflow Interrupt - interrupt every 0.01 sec
        SECTION "Org $50",HOME[$50]
        di
        push    af
        push    bc
        push    hl
        call    service_timer_int
        pop     hl
        pop     bc
        pop     af
        reti


        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      Startup

        NINTENDO_LOGO           ;Nintendo graphic logo

;Rom Header Info

        ;0123456789ABCDE
DB      "GBSoundXpermntr"       ;Cart name 15bytes
DB      $00                     ;not color gb
DB      0,0,0                   ;Not used
DB      CART_ROM                ;Cart type
DB      CART_ROM_256K           ;ROM Size (in bits)
DB      CART_RAM_NONE           ;RAM Size (in bits)
DB      0,0                     ;Maker ID
DB      0                       ;Version     =0
DB      0                       ;Complement check (important)
DW      0                       ;Checksum (not important)

        SECTION "Main",home[$0150]

Startup
        call    initialize

Main
        halt            ;reduces power to 68% if halt is used
        nop

        call    button_actions
        jr      Main

service_vblank_int
        ld      a,[timediv]
        add     $01
        ld      [timediv],a
        cp      blinktime
        jr      nz,not_blink
        xor     a
        ld      [timediv],a
        ld      a,[blink]
        xor     $FF
        ld      [blink],a

not_blink
        call    write_to_screen

        ld      a,[NR_update]
        add     $01
        cp      $08
        jr      nz,store_NR
        xor     a

store_NR
        ld      [NR_update],a

        ret

service_timer_int
        call    button_read

        ret

button_read
        ld      a,P1F_5
        ld      [rP1],a      ;turn on P15

        ld      a,[rP1]      ;delay
        ld      a,[rP1]
        cpl
        and     $0F
        swap    a
        ld      b,a

        ld      a,P1F_4
        ld      [rP1],a      ;turn on P14
        ld      a,[rP1]      ;delay
        ld      a,[rP1]
        ld      a,[rP1]    
        ld      a,[rP1]
        ld      a,[rP1]    
        ld      a,[rP1]
        cpl
        and     $0F
        or      b
        ld      b,a

        ld      a,[button_data]
        xor     a,b
        ld      [button_data_chg],a
        ld      a,b
        ld      [button_data],a

        ld      a,P1F_5|P1F_4
        ld      [rP1],a

check_UP
        ld      a,[button_data_chg];see if UP changed
        bit     PADB_UP,a
        jr      z,stable_UP

        ld      a,$03
        ld      [UP_debounce],a
        jr      check_DOWN

stable_UP
        ld      a,[UP_debounce]
        cp      $00
        jr      z,check_DOWN    ;not debouncing
        sub     $01
        ld      [UP_debounce],a
        jr      nz,check_DOWN   ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_UP,a
        jr      z,clear_UP

        ld      a,[button_debounced]
        set     PADB_UP,a
        ld      [button_debounced],a
        jr      check_DOWN

clear_UP
        ld      a,[button_debounced]
        res     PADB_UP,a
        ld      [button_debounced],a

check_DOWN
        ld      a,[button_data_chg];see if DOWN changed
        bit     PADB_DOWN,a
        jr      z,stable_DOWN

        ld      a,$03
        ld      [DOWN_debounce],a
        jr      check_LEFT

stable_DOWN
        ld      a,[DOWN_debounce]
        cp      $00
        jr      z,check_LEFT    ;not debouncing
        sub     $01
        ld      [DOWN_debounce],a
        jr      nz,check_LEFT   ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_DOWN,a
        jr      z,clear_DOWN

        ld      a,[button_debounced]
        set     PADB_DOWN,a
        ld      [button_debounced],a
        jr      check_LEFT

clear_DOWN
        ld      a,[button_debounced]
        res     PADB_DOWN,a
        ld      [button_debounced],a

check_LEFT
        ld      a,[button_data_chg];see if LEFT changed
        bit     PADB_LEFT,a
        jr      z,stable_LEFT

        ld      a,$03
        ld      [LEFT_debounce],a
        jr      check_RIGHT

stable_LEFT
        ld      a,[LEFT_debounce]
        cp      $00
        jr      z,check_RIGHT   ;not debouncing
        sub     $01
        ld      [LEFT_debounce],a
        jr      nz,check_RIGHT  ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_LEFT,a
        jr      z,clear_LEFT

        ld      a,[button_debounced]
        set     PADB_LEFT,a
        ld      [button_debounced],a
        jr      check_RIGHT

clear_LEFT
        ld      a,[button_debounced]
        res     PADB_LEFT,a
        ld      [button_debounced],a

check_RIGHT
        ld      a,[button_data_chg];see if RIGHT changed
        bit     PADB_RIGHT,a
        jr      z,stable_RIGHT

        ld      a,$03
        ld      [RIGHT_debounce],a
        jr      check_SELECT

stable_RIGHT
        ld      a,[RIGHT_debounce]
        cp      $00
        jr      z,check_SELECT  ;not debouncing
        sub     $01
        ld      [RIGHT_debounce],a
        jr      nz,check_SELECT ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_RIGHT,a
        jr      z,clear_RIGHT

        ld      a,[button_debounced]
        set     PADB_RIGHT,a
        ld      [button_debounced],a
        jr      check_SELECT

clear_RIGHT
        ld      a,[button_debounced]
        res     PADB_RIGHT,a
        ld      [button_debounced],a

check_SELECT
        ld      a,[button_data_chg];see if SELECT changed
        bit     PADB_SELECT,a
        jr      z,stable_SELECT

        ld      a,$03
        ld      [SELECT_debounce],a
        jr      check_START

stable_SELECT
        ld      a,[SELECT_debounce]
        cp      $00
        jr      z,check_START   ;not debouncing
        sub     $01
        ld      [SELECT_debounce],a
        jr      nz,check_START  ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_SELECT,a
        jr      z,clear_SELECT

        ld      a,[button_debounced]
        set     PADB_SELECT,a
        ld      [button_debounced],a
        jr      check_START

clear_SELECT
        ld      a,[button_debounced]
        res     PADB_SELECT,a
        ld      [button_debounced],a

check_START
        ld      a,[button_data_chg];see if START changed
        bit     PADB_START,a
        jr      z,stable_START

        ld      a,$03
        ld      [START_debounce],a
        jr      check_A

stable_START
        ld      a,[START_debounce]
        cp      $00
        jr      z,check_A       ;not debouncing
        sub     $01
        ld      [START_debounce],a
        jr      nz,check_A      ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_START,a
        jr      z,clear_START

        ld      a,[button_debounced]
        set     PADB_START,a
        ld      [button_debounced],a
        jr      check_A

clear_START
        ld      a,[button_debounced]
        res     PADB_START,a
        ld      [button_debounced],a

check_A
        ld      a,[button_data_chg];see if A changed
        bit     PADB_A,a
        jr      z,stable_A

        ld      a,$03
        ld      [A_debounce],a
        jr      check_B

stable_A
        ld      a,[A_debounce]
        cp      $00
        jr      z,check_B       ;not debouncing
        sub     $01
        ld      [A_debounce],a
        jr      nz,check_B      ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_A,a
        jr      z,clear_A

        ld      a,[button_debounced]
        set     PADB_A,a
        ld      [button_debounced],a
        jr      check_B

clear_A
        ld      a,[button_debounced]
        res     PADB_A,a
        ld      [button_debounced],a

check_B
        ld      a,[button_data_chg];see if B changed
        bit     PADB_B,a
        jr      z,stable_B

        ld      a,$03
        ld      [B_debounce],a
        jr      check_done

stable_B
        ld      a,[B_debounce]
        cp      $00
        jr      z,check_done    ;not debouncing
        sub     $01
        ld      [B_debounce],a
        jr      nz,check_done   ;debounced when = 0

        ld      a,[button_data]
        bit     PADB_B,a
        jr      z,clear_B

        ld      a,[button_debounced]
        set     PADB_B,a
        ld      [button_debounced],a
        jr      check_done

clear_B
        ld      a,[button_debounced]
        res     PADB_B,a
        ld      [button_debounced],a

check_done
        ld      a,$30
        ldh     [$00],a

        ret

button_actions
        ld      a,[button_debounced]
        bit     PADB_RIGHT,a
        jp      z,not_RIGHT

        res     PADB_RIGHT,a
        ld      [button_debounced],a

chk_RIGHT_00
        ld      a,[FirstLine]
        cp      $00
        jr      nz,chk_RIGHT_01
        ld      a,[cursor]
        cp      $12
        jr      nz,chk_RIGHT_01
        ld      a,$01
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_01
        ld      a,[FirstLine]
        cp      $01
        jr      nz,chk_RIGHT_02
        ld      a,[cursor]
        cp      $13
        jr      nz,chk_RIGHT_02
        ld      a,$02
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_02
        ld      a,[FirstLine]
        cp      $02
        jr      nz,chk_RIGHT_03
        ld      a,[cursor]
        cp      $17
        jr      nz,chk_RIGHT_03
        ld      a,$03
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_03
        ld      a,[FirstLine]
        cp      $03
        jr      nz,chk_RIGHT_04
        ld      a,[cursor]
        cp      $19
        jr      nz,chk_RIGHT_04
        ld      a,$04
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_04
        ld      a,[FirstLine]
        cp      $04
        jr      nz,chk_RIGHT_05
        ld      a,[cursor]
        cp      $1A
        jr      nz,chk_RIGHT_05
        ld      a,$05
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_05
        ld      a,[FirstLine]
        cp      $05
        jr      nz,chk_RIGHT_06
        ld      a,[cursor]
        cp      $1D
        jr      nz,chk_RIGHT_06
        ld      a,$06
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_06
        ld      a,[FirstLine]
        cp      $06
        jr      nz,chk_RIGHT_07
        ld      a,[cursor]
        cp      $1E
        jr      nz,chk_RIGHT_07
        ld      a,$07
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_07
        ld      a,[FirstLine]
        cp      $07
        jr      nz,chk_RIGHT_08
        ld      a,[cursor]
        cp      $22
        jr      nz,chk_RIGHT_08
        ld      a,$08
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_08
        ld      a,[FirstLine]
        cp      $08
        jr      nz,chk_RIGHT_09
        ld      a,[cursor]
        cp      $24
        jr      nz,chk_RIGHT_09
        ld      a,$09
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_09
        ld      a,[FirstLine]
        cp      $09
        jr      nz,chk_RIGHT_0A
        ld      a,[cursor]
        cp      $27
        jr      nz,chk_RIGHT_0A
        ld      a,$0A
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_0A
        ld      a,[FirstLine]
        cp      $0A
        jr      nz,chk_RIGHT_0B
        ld      a,[cursor]
        cp      $2A
        jr      nz,chk_RIGHT_0B
        ld      a,$0B
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_0B
        ld      a,[FirstLine]
        cp      $0B
        jr      nz,chk_RIGHT_0C
        ld      a,[cursor]
        cp      $2D
        jr      nz,chk_RIGHT_0C
        ld      a,$0C
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_0C
        ld      a,[FirstLine]
        cp      $0C
        jr      nz,chk_RIGHT_0D
        ld      a,[cursor]
        cp      $31
        jr      nz,chk_RIGHT_0D
        ld      a,$0D
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_0D
        ld      a,[FirstLine]
        cp      $0D
        jr      nz,chk_RIGHT_0E
        ld      a,[cursor]
        cp      $39
        jr      nz,chk_RIGHT_0E
        ld      a,$0E
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_0E
        ld      a,[FirstLine]
        cp      $0E
        jr      nz,chk_RIGHT_0F
        ld      a,[cursor]
        cp      $3B
        jr      nz,chk_RIGHT_0F
        ld      a,$0F
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_0F
        ld      a,[FirstLine]
        cp      $0F
        jr      nz,chk_RIGHT_10
        ld      a,[cursor]
        cp      $3D
        jr      nz,chk_RIGHT_10
        ld      a,$10
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_10
        ld      a,[FirstLine]
        cp      $10
        jr      nz,chk_RIGHT_11
        ld      a,[cursor]
        cp      $3F
        jr      nz,chk_RIGHT_11
        ld      a,$11
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_11
        ld      a,[FirstLine]
        cp      $11
        jr      nz,chk_RIGHT_12
        ld      a,[cursor]
        cp      $41
        jr      nz,chk_RIGHT_12
        ld      a,$12
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_12
        ld      a,[FirstLine]
        cp      $12
        jr      nz,chk_RIGHT_13
        ld      a,[cursor]
        cp      $43
        jr      nz,chk_RIGHT_13
        ld      a,$13
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_13
        ld      a,[FirstLine]
        cp      $13
        jr      nz,chk_RIGHT_14
        ld      a,[cursor]
        cp      $45
        jr      nz,chk_RIGHT_14
        ld      a,$14
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_14
        ld      a,[FirstLine]
        cp      $14
        jr      nz,chk_RIGHT_15
        ld      a,[cursor]
        cp      $47
        jr      nz,chk_RIGHT_15
        ld      a,$15
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_15
        ld      a,[FirstLine]
        cp      $15
        jr      nz,chk_RIGHT_16
        ld      a,[cursor]
        cp      $49
        jr      nz,chk_RIGHT_16
        ld      a,$16
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_16
        ld      a,[FirstLine]
        cp      $16
        jr      nz,chk_RIGHT_17
        ld      a,[cursor]
        cp      $4B
        jr      nz,chk_RIGHT_17
        ld      a,$17
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_17
        ld      a,[FirstLine]
        cp      $17
        jr      nz,chk_RIGHT_18
        ld      a,[cursor]
        cp      $4D
        jr      nz,chk_RIGHT_18
        ld      a,$18
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_18
        ld      a,[FirstLine]
        cp      $18
        jr      nz,chk_RIGHT_19
        ld      a,[cursor]
        cp      $4F
        jr      nz,chk_RIGHT_19
        ld      a,$19
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_19
        ld      a,[FirstLine]
        cp      $19
        jr      nz,chk_RIGHT_1A
        ld      a,[cursor]
        cp      $51
        jr      nz,chk_RIGHT_1A
        ld      a,$1A
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_1A
        ld      a,[FirstLine]
        cp      $1A
        jr      nz,chk_RIGHT_1B
        ld      a,[cursor]
        cp      $53
        jr      nz,chk_RIGHT_1B
        ld      a,$1B
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_1B
        ld      a,[FirstLine]
        cp      $1B
        jr      nz,chk_RIGHT_1C
        ld      a,[cursor]
        cp      $55
        jr      nz,chk_RIGHT_1C
        ld      a,$1C
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_1C
        ld      a,[FirstLine]
        cp      $1C
        jr      nz,chk_RIGHT_1D
        ld      a,[cursor]
        cp      $57
        jr      nz,chk_RIGHT_1D
        ld      a,$1D
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_1D
        ld      a,[FirstLine]
        cp      $1D
        jr      nz,chk_RIGHT_1E
        ld      a,[cursor]
        cp      $59
        jr      nz,chk_RIGHT_1E
        ld      a,$1E
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_1E
        ld      a,[FirstLine]
        cp      $1E
        jr      nz,chk_RIGHT_1F
        ld      a,[cursor]
        cp      $5B
        jr      nz,chk_RIGHT_1F
        ld      a,$1F
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_1F
        ld      a,[FirstLine]
        cp      $1F
        jr      nz,chk_RIGHT_20
        ld      a,[cursor]
        cp      $03
        jr      nz,chk_RIGHT_20
        ld      a,$20
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_20
        ld      a,[FirstLine]
        cp      $20
        jr      nz,chk_RIGHT_21
        ld      a,[cursor]
        cp      $05
        jr      nz,chk_RIGHT_21
        ld      a,$21
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_21
        ld      a,[FirstLine]
        cp      $21
        jr      nz,chk_RIGHT_22
        ld      a,[cursor]
        cp      $08
        jr      nz,chk_RIGHT_22
        ld      a,$22
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_22
        ld      a,[FirstLine]
        cp      $22
        jr      nz,chk_RIGHT_23
        ld      a,[cursor]
        cp      $09
        jr      nz,chk_RIGHT_23
        ld      a,$23
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_23
        ld      a,[FirstLine]
        cp      $23
        jr      nz,chk_RIGHT_24
        ld      a,[cursor]
        cp      $0D
        jr      nz,chk_RIGHT_24
        ld      a,$24
        ld      [FirstLine],a
        jp      chk_RIGHT_done

chk_RIGHT_24
        ld      a,[FirstLine]
        cp      $24
        jr      nz,chk_RIGHT_done
        ld      a,[cursor]
        cp      $0F
        jr      nz,chk_RIGHT_done
        xor     a
        ld      [FirstLine],a

chk_RIGHT_done
        ld      a,[cursor]
        add     $01
        cp      last_cursor+1
        jr      nz,no_set_min
        xor     a

no_set_min
        ld      [cursor],a

        xor     a
        ld      [blink],a

not_RIGHT
        ld      a,[button_debounced]
        bit     PADB_LEFT,a
        jp      z,not_LEFT

        res     PADB_LEFT,a
        ld      [button_debounced],a

chk_LEFT_00
        ld      a,[FirstLine]
        cp      $00
        jr      nz,chk_LEFT_01
        ld      a,[cursor]
        cp      $00
        jr      nz,chk_LEFT_01
        ld      a,last_line
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_01
        ld      a,[FirstLine]
        cp      $01
        jr      nz,chk_LEFT_02
        ld      a,[cursor]
        cp      $04
        jr      nz,chk_LEFT_02
        xor     a
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_02
        ld      a,[FirstLine]
        cp      $02
        jr      nz,chk_LEFT_03
        ld      a,[cursor]
        cp      $06
        jr      nz,chk_LEFT_03
        ld      a,$01
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_03
        ld      a,[FirstLine]
        cp      $03
        jr      nz,chk_LEFT_04
        ld      a,[cursor]
        cp      $09
        jr      nz,chk_LEFT_04
        ld      a,$02
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_04
        ld      a,[FirstLine]
        cp      $04
        jr      nz,chk_LEFT_05
        ld      a,[cursor]
        cp      $0A
        jr      nz,chk_LEFT_05
        ld      a,$03
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_05
        ld      a,[FirstLine]
        cp      $05
        jr      nz,chk_LEFT_06
        ld      a,[cursor]
        cp      $0E
        jr      nz,chk_LEFT_06
        ld      a,$04
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_06
        ld      a,[FirstLine]
        cp      $06
        jr      nz,chk_LEFT_07
        ld      a,[cursor]
        cp      $10
        jr      nz,chk_LEFT_07
        ld      a,$05
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_07
        ld      a,[FirstLine]
        cp      $07
        jr      nz,chk_LEFT_08
        ld      a,[cursor]
        cp      $13
        jr      nz,chk_LEFT_08
        ld      a,$06
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_08
        ld      a,[FirstLine]
        cp      $08
        jr      nz,chk_LEFT_09
        ld      a,[cursor]
        cp      $14
        jr      nz,chk_LEFT_09
        ld      a,$07
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_09
        ld      a,[FirstLine]
        cp      $09
        jr      nz,chk_LEFT_0A
        ld      a,[cursor]
        cp      $18
        jr      nz,chk_LEFT_0A
        ld      a,$08
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_0A
        ld      a,[FirstLine]
        cp      $0A
        jr      nz,chk_LEFT_0B
        ld      a,[cursor]
        cp      $1A
        jr      nz,chk_LEFT_0B
        ld      a,$09
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_0B
        ld      a,[FirstLine]
        cp      $0B
        jr      nz,chk_LEFT_0C
        ld      a,[cursor]
        cp      $1B
        jr      nz,chk_LEFT_0C
        ld      a,$0A
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_0C
        ld      a,[FirstLine]
        cp      $0C
        jr      nz,chk_LEFT_0D
        ld      a,[cursor]
        cp      $1E
        jr      nz,chk_LEFT_0D
        ld      a,$0B
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_0D
        ld      a,[FirstLine]
        cp      $0D
        jr      nz,chk_LEFT_0E
        ld      a,[cursor]
        cp      $1F
        jr      nz,chk_LEFT_0E
        ld      a,$0C
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_0E
        ld      a,[FirstLine]
        cp      $0E
        jr      nz,chk_LEFT_0F
        ld      a,[cursor]
        cp      $23
        jr      nz,chk_LEFT_0F
        ld      a,$0D
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_0F
        ld      a,[FirstLine]
        cp      $0F
        jr      nz,chk_LEFT_10
        ld      a,[cursor]
        cp      $25
        jr      nz,chk_LEFT_10
        ld      a,$0E
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_10
        ld      a,[FirstLine]
        cp      $10
        jr      nz,chk_LEFT_11
        ld      a,[cursor]
        cp      $28
        jr      nz,chk_LEFT_11
        ld      a,$0F
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_11
        ld      a,[FirstLine]
        cp      $11
        jr      nz,chk_LEFT_12
        ld      a,[cursor]
        cp      $2B
        jr      nz,chk_LEFT_12
        ld      a,$10
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_12
        ld      a,[FirstLine]
        cp      $12
        jr      nz,chk_LEFT_13
        ld      a,[cursor]
        cp      $2E
        jr      nz,chk_LEFT_13
        ld      a,$11
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_13
        ld      a,[FirstLine]
        cp      $13
        jr      nz,chk_LEFT_14
        ld      a,[cursor]
        cp      $32
        jr      nz,chk_LEFT_14
        ld      a,$12
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_14
        ld      a,[FirstLine]
        cp      $14
        jr      nz,chk_LEFT_15
        ld      a,[cursor]
        cp      $3A
        jr      nz,chk_LEFT_15
        ld      a,$13
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_15
        ld      a,[FirstLine]
        cp      $15
        jr      nz,chk_LEFT_16
        ld      a,[cursor]
        cp      $3C
        jr      nz,chk_LEFT_16
        ld      a,$14
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_16
        ld      a,[FirstLine]
        cp      $16
        jr      nz,chk_LEFT_17
        ld      a,[cursor]
        cp      $3E
        jr      nz,chk_LEFT_17
        ld      a,$15
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_17
        ld      a,[FirstLine]
        cp      $17
        jr      nz,chk_LEFT_18
        ld      a,[cursor]
        cp      $40
        jr      nz,chk_LEFT_18
        ld      a,$16
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_18
        ld      a,[FirstLine]
        cp      $18
        jr      nz,chk_LEFT_19
        ld      a,[cursor]
        cp      $42
        jr      nz,chk_LEFT_19
        ld      a,$17
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_19
        ld      a,[FirstLine]
        cp      $19
        jr      nz,chk_LEFT_1A
        ld      a,[cursor]
        cp      $44
        jr      nz,chk_LEFT_1A
        ld      a,$18
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_1A
        ld      a,[FirstLine]
        cp      $1A
        jr      nz,chk_LEFT_1B
        ld      a,[cursor]
        cp      $46
        jr      nz,chk_LEFT_1B
        ld      a,$19
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_1B
        ld      a,[FirstLine]
        cp      $1B
        jr      nz,chk_LEFT_1C
        ld      a,[cursor]
        cp      $48
        jr      nz,chk_LEFT_1C
        ld      a,$1A
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_1C
        ld      a,[FirstLine]
        cp      $1C
        jr      nz,chk_LEFT_1D
        ld      a,[cursor]
        cp      $4A
        jr      nz,chk_LEFT_1D
        ld      a,$1B
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_1D
        ld      a,[FirstLine]
        cp      $1D
        jr      nz,chk_LEFT_1E
        ld      a,[cursor]
        cp      $4C
        jr      nz,chk_LEFT_1E
        ld      a,$1C
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_1E
        ld      a,[FirstLine]
        cp      $1E
        jr      nz,chk_LEFT_1F
        ld      a,[cursor]
        cp      $4E
        jr      nz,chk_LEFT_1F
        ld      a,$1D
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_1F
        ld      a,[FirstLine]
        cp      $1F
        jr      nz,chk_LEFT_20
        ld      a,[cursor]
        cp      $50
        jr      nz,chk_LEFT_20
        ld      a,$1E
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_20
        ld      a,[FirstLine]
        cp      $20
        jr      nz,chk_LEFT_21
        ld      a,[cursor]
        cp      $52
        jr      nz,chk_LEFT_21
        ld      a,$1F
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_21
        ld      a,[FirstLine]
        cp      $21
        jr      nz,chk_LEFT_22
        ld      a,[cursor]
        cp      $54
        jr      nz,chk_LEFT_22
        ld      a,$20
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_22
        ld      a,[FirstLine]
        cp      $22
        jr      nz,chk_LEFT_23
        ld      a,[cursor]
        cp      $56
        jr      nz,chk_LEFT_23
        ld      a,$21
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_23
        ld      a,[FirstLine]
        cp      $23
        jr      nz,chk_LEFT_24
        ld      a,[cursor]
        cp      $58
        jr      nz,chk_LEFT_24
        ld      a,$22
        ld      [FirstLine],a
        jp      chk_LEFT_done

chk_LEFT_24
        ld      a,[FirstLine]
        cp      $24
        jr      nz,chk_LEFT_done
        ld      a,[cursor]
        cp      $5A
        jr      nz,chk_LEFT_done
        ld      a,$23
        ld      [FirstLine],a

chk_LEFT_done
        ld      a,[cursor]
        sub     $01
        cp      $FF
        jr      nz,no_set_max
        ld      a,last_cursor

no_set_max
        ld      [cursor],a

        xor     a
        ld      [blink],a

not_LEFT
        ld      a,[button_debounced]
        bit     PADB_DOWN,a
        jp      z,not_DOWN

        res     PADB_DOWN,a
        ld      [button_debounced],a

        ld      a,[cursor]
        cp      $00
        jr      nz,not_dec_00
        ld      a,[val_FF10]
        and     $7F
        ld      b,a
        ld      a,[val_FF10]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF10],a

not_dec_00
        ld      a,[cursor]
        cp      $01
        jr      nz,not_dec_01
        ld      a,[val_FF10]
        and     $8F
        ld      b,a
        ld      a,[val_FF10]
        and     $70
        sub     $10
        and     $70
        add     b
        ld      [val_FF10],a

not_dec_01
        ld      a,[cursor]
        cp      $02
        jr      nz,not_dec_02
        ld      a,[val_FF10]
        and     $F7
        ld      b,a
        ld      a,[val_FF10]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF10],a

not_dec_02
        ld      a,[cursor]
        cp      $03
        jr      nz,not_dec_03
        ld      a,[val_FF10]
        and     $F8
        ld      b,a
        ld      a,[val_FF10]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF10],a

not_dec_03
        ld      a,[cursor]
        cp      $04
        jr      nz,not_dec_04
        ld      a,[val_FF11]
        and     $3F
        ld      b,a
        ld      a,[val_FF11]
        and     $C0
        sub     $40
        and     $C0
        add     b
        ld      [val_FF11],a

not_dec_04
        ld      a,[cursor]
        cp      $05
        jr      nz,not_dec_05
        ld      a,[val_FF11]
        and     $C0
        ld      b,a
        ld      a,[val_FF11]
        and     $3F
        sub     $01
        and     $3F
        add     b
        ld      [val_FF11],a

not_dec_05
        ld      a,[cursor]
        cp      $06
        jr      nz,not_dec_06
        ld      a,[val_FF12]
        and     $0F
        ld      b,a
        ld      a,[val_FF12]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF12],a

not_dec_06
        ld      a,[cursor]
        cp      $07
        jr      nz,not_dec_07
        ld      a,[val_FF12]
        and     $F7
        ld      b,a
        ld      a,[val_FF12]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF12],a

not_dec_07
        ld      a,[cursor]
        cp      $08
        jr      nz,not_dec_08
        ld      a,[val_FF12]
        and     $F8
        ld      b,a
        ld      a,[val_FF12]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF12],a

not_dec_08
        ld      a,[cursor]
        cp      $09
        jr      nz,not_dec_09
        ld      a,[val_FF13]
        sub     $01
        ld      [val_FF13],a

not_dec_09
        ld      a,[cursor]
        cp      $0A
        jr      nz,not_dec_0A
        ld      a,[val_FF14]
        and     $7F
        ld      b,a
        ld      a,[val_FF14]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF14],a

not_dec_0A
        ld      a,[cursor]
        cp      $0B
        jr      nz,not_dec_0B
        ld      a,[val_FF14]
        and     $BF
        ld      b,a
        ld      a,[val_FF14]
        and     $40
        sub     $40
        and     $40
        add     b
        ld      [val_FF14],a

not_dec_0B
        ld      a,[cursor]
        cp      $0C
        jr      nz,not_dec_0C
        ld      a,[val_FF14]
        and     $C7
        ld      b,a
        ld      a,[val_FF14]
        and     $38
        sub     $08
        and     $38
        add     b
        ld      [val_FF14],a

not_dec_0C
        ld      a,[cursor]
        cp      $0D
        jr      nz,not_dec_0D
        ld      a,[val_FF14]
        and     $F8
        ld      b,a
        ld      a,[val_FF14]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF14],a

not_dec_0D
        ld      a,[cursor]
        cp      $0E
        jr      nz,not_dec_0E
        ld      a,[val_FF16]
        and     $3F
        ld      b,a
        ld      a,[val_FF16]
        and     $C0
        sub     $40
        and     $C0
        add     b
        ld      [val_FF16],a

not_dec_0E
        ld      a,[cursor]
        cp      $0F
        jr      nz,not_dec_0F
        ld      a,[val_FF16]
        and     $C0
        ld      b,a
        ld      a,[val_FF16]
        and     $3F
        sub     $01
        and     $3F
        add     b
        ld      [val_FF16],a

not_dec_0F
        ld      a,[cursor]
        cp      $10
        jr      nz,not_dec_10
        ld      a,[val_FF17]
        and     $0F
        ld      b,a
        ld      a,[val_FF17]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF17],a

not_dec_10
        ld      a,[cursor]
        cp      $11
        jr      nz,not_dec_11
        ld      a,[val_FF17]
        and     $F7
        ld      b,a
        ld      a,[val_FF17]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF17],a

not_dec_11
        ld      a,[cursor]
        cp      $12
        jr      nz,not_dec_12
        ld      a,[val_FF17]
        and     $F8
        ld      b,a
        ld      a,[val_FF17]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF17],a

not_dec_12
        ld      a,[cursor]
        cp      $13
        jr      nz,not_dec_13
        ld      a,[val_FF18]
        sub     $01
        ld      [val_FF18],a

not_dec_13
        ld      a,[cursor]
        cp      $14
        jr      nz,not_dec_14
        ld      a,[val_FF19]
        and     $7F
        ld      b,a
        ld      a,[val_FF19]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF19],a

not_dec_14
        ld      a,[cursor]
        cp      $15
        jr      nz,not_dec_15
        ld      a,[val_FF19]
        and     $BF
        ld      b,a
        ld      a,[val_FF19]
        and     $40
        sub     $40
        and     $40
        add     b
        ld      [val_FF19],a

not_dec_15
        ld      a,[cursor]
        cp      $16
        jr      nz,not_dec_16
        ld      a,[val_FF19]
        and     $C7
        ld      b,a
        ld      a,[val_FF19]
        and     $38
        sub     $08
        and     $38
        add     b
        ld      [val_FF19],a

not_dec_16
        ld      a,[cursor]
        cp      $17
        jr      nz,not_dec_17
        ld      a,[val_FF19]
        and     $F8
        ld      b,a
        ld      a,[val_FF19]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF19],a

not_dec_17
        ld      a,[cursor]
        cp      $18
        jr      nz,not_dec_18
        ld      a,[val_FF1A]
        and     $7F
        ld      b,a
        ld      a,[val_FF1A]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF1A],a

not_dec_18
        ld      a,[cursor]
        cp      $19
        jr      nz,not_dec_19
        ld      a,[val_FF1A]
        and     $80
        ld      b,a
        ld      a,[val_FF1A]
        and     $7F
        sub     $01
        and     $7F
        add     b
        ld      [val_FF1A],a

not_dec_19
        ld      a,[cursor]
        cp      $1A
        jr      nz,not_dec_1A
        ld      a,[val_FF1B]
        sub     $01
        ld      [val_FF1B],a

not_dec_1A
        ld      a,[cursor]
        cp      $1B
        jr      nz,not_dec_1B
        ld      a,[val_FF1C]
        and     $7F
        ld      b,a
        ld      a,[val_FF1C]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF1C],a

not_dec_1B
        ld      a,[cursor]
        cp      $1C
        jr      nz,not_dec_1C
        ld      a,[val_FF1C]
        and     $9F
        ld      b,a
        ld      a,[val_FF1C]
        and     $60
        sub     $20
        and     $60
        add     b
        ld      [val_FF1C],a

not_dec_1C
        ld      a,[cursor]
        cp      $1D
        jr      nz,not_dec_1D
        ld      a,[val_FF1C]
        and     $E0
        ld      b,a
        ld      a,[val_FF1C]
        and     $1F
        sub     $01
        and     $1F
        add     b
        ld      [val_FF1C],a

not_dec_1D
        ld      a,[cursor]
        cp      $1E
        jr      nz,not_dec_1E
        ld      a,[val_FF1D]
        sub     $01
        ld      [val_FF1D],a

not_dec_1E
        ld      a,[cursor]
        cp      $1F
        jr      nz,not_dec_1F
        ld      a,[val_FF1E]
        and     $7F
        ld      b,a
        ld      a,[val_FF1E]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF1E],a

not_dec_1F
        ld      a,[cursor]
        cp      $20
        jr      nz,not_dec_20
        ld      a,[val_FF1E]
        and     $BF
        ld      b,a
        ld      a,[val_FF1E]
        and     $40
        sub     $40
        and     $40
        add     b
        ld      [val_FF1E],a

not_dec_20
        ld      a,[cursor]
        cp      $21
        jr      nz,not_dec_21
        ld      a,[val_FF1E]
        and     $C7
        ld      b,a
        ld      a,[val_FF1E]
        and     $38
        sub     $08
        and     $38
        add     b
        ld      [val_FF1E],a

not_dec_21
        ld      a,[cursor]
        cp      $22
        jr      nz,not_dec_22
        ld      a,[val_FF1E]
        and     $F8
        ld      b,a
        ld      a,[val_FF1E]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF1E],a

not_dec_22
        ld      a,[cursor]
        cp      $23
        jr      nz,not_dec_23
        ld      a,[val_FF20]
        and     $3F
        ld      b,a
        ld      a,[val_FF20]
        and     $C0
        sub     $40
        and     $C0
        add     b
        ld      [val_FF20],a

not_dec_23
        ld      a,[cursor]
        cp      $24
        jr      nz,not_dec_24
        ld      a,[val_FF20]
        and     $C0
        ld      b,a
        ld      a,[val_FF20]
        and     $3F
        sub     $01
        and     $3F
        add     b
        ld      [val_FF20],a

not_dec_24
        ld      a,[cursor]
        cp      $25
        jr      nz,not_dec_25
        ld      a,[val_FF21]
        and     $0F
        ld      b,a
        ld      a,[val_FF21]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF21],a

not_dec_25
        ld      a,[cursor]
        cp      $26
        jr      nz,not_dec_26
        ld      a,[val_FF21]
        and     $F7
        ld      b,a
        ld      a,[val_FF21]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF21],a

not_dec_26
        ld      a,[cursor]
        cp      $27
        jr      nz,not_dec_27
        ld      a,[val_FF21]
        and     $F8
        ld      b,a
        ld      a,[val_FF21]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF21],a

not_dec_27
        ld      a,[cursor]
        cp      $28
        jr      nz,not_dec_28
        ld      a,[val_FF22]
        and     $0F
        ld      b,a
        ld      a,[val_FF22]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF22],a

not_dec_28
        ld      a,[cursor]
        cp      $29
        jr      nz,not_dec_29
        ld      a,[val_FF22]
        and     $F7
        ld      b,a
        ld      a,[val_FF22]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF22],a

not_dec_29
        ld      a,[cursor]
        cp      $2A
        jr      nz,not_dec_2A
        ld      a,[val_FF22]
        and     $F8
        ld      b,a
        ld      a,[val_FF22]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF22],a

not_dec_2A
        ld      a,[cursor]
        cp      $2B
        jr      nz,not_dec_2B
        ld      a,[val_FF23]
        and     $7F
        ld      b,a
        ld      a,[val_FF23]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF23],a

not_dec_2B
        ld      a,[cursor]
        cp      $2C
        jr      nz,not_dec_2C
        ld      a,[val_FF23]
        and     $BF
        ld      b,a
        ld      a,[val_FF23]
        and     $40
        sub     $40
        and     $40
        add     b
        ld      [val_FF23],a

not_dec_2C
        ld      a,[cursor]
        cp      $2D
        jr      nz,not_dec_2D
        ld      a,[val_FF23]
        and     $C0
        ld      b,a
        ld      a,[val_FF23]
        and     $3F
        sub     $01
        and     $3F
        add     b
        ld      [val_FF23],a

not_dec_2D
        ld      a,[cursor]
        cp      $2E
        jr      nz,not_dec_2E
        ld      a,[val_FF24]
        and     $7F
        ld      b,a
        ld      a,[val_FF24]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF24],a

not_dec_2E
        ld      a,[cursor]
        cp      $2F
        jr      nz,not_dec_2F
        ld      a,[val_FF24]
        and     $8F
        ld      b,a
        ld      a,[val_FF24]
        and     $70
        sub     $10
        and     $70
        add     b
        ld      [val_FF24],a

not_dec_2F
        ld      a,[cursor]
        cp      $30
        jr      nz,not_dec_30
        ld      a,[val_FF24]
        and     $F7
        ld      b,a
        ld      a,[val_FF24]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF24],a

not_dec_30
        ld      a,[cursor]
        cp      $31
        jr      nz,not_dec_31
        ld      a,[val_FF24]
        and     $F8
        ld      b,a
        ld      a,[val_FF24]
        and     $07
        sub     $01
        and     $07
        add     b
        ld      [val_FF24],a

not_dec_31
        ld      a,[cursor]
        cp      $32
        jr      nz,not_dec_32
        ld      a,[val_FF25]
        and     $7F
        ld      b,a
        ld      a,[val_FF25]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF25],a

not_dec_32
        ld      a,[cursor]
        cp      $33
        jr      nz,not_dec_33
        ld      a,[val_FF25]
        and     $BF
        ld      b,a
        ld      a,[val_FF25]
        and     $40
        sub     $40
        and     $40
        add     b
        ld      [val_FF25],a

not_dec_33
        ld      a,[cursor]
        cp      $34
        jr      nz,not_dec_34
        ld      a,[val_FF25]
        and     $DF
        ld      b,a
        ld      a,[val_FF25]
        and     $20
        sub     $20
        and     $20
        add     b
        ld      [val_FF25],a

not_dec_34
        ld      a,[cursor]
        cp      $35
        jr      nz,not_dec_35
        ld      a,[val_FF25]
        and     $EF
        ld      b,a
        ld      a,[val_FF25]
        and     $10
        sub     $10
        and     $10
        add     b
        ld      [val_FF25],a

not_dec_35
        ld      a,[cursor]
        cp      $36
        jr      nz,not_dec_36
        ld      a,[val_FF25]
        and     $F7
        ld      b,a
        ld      a,[val_FF25]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF25],a

not_dec_36
        ld      a,[cursor]
        cp      $37
        jr      nz,not_dec_37
        ld      a,[val_FF25]
        and     $FB
        ld      b,a
        ld      a,[val_FF25]
        and     $04
        sub     $04
        and     $04
        add     b
        ld      [val_FF25],a

not_dec_37
        ld      a,[cursor]
        cp      $38
        jr      nz,not_dec_38
        ld      a,[val_FF25]
        and     $FD
        ld      b,a
        ld      a,[val_FF25]
        and     $02
        sub     $02
        and     $02
        add     b
        ld      [val_FF25],a

not_dec_38
        ld      a,[cursor]
        cp      $39
        jr      nz,not_dec_39
        ld      a,[val_FF25]
        and     $FE
        ld      b,a
        ld      a,[val_FF25]
        and     $01
        sub     $01
        and     $01
        add     b
        ld      [val_FF25],a

not_dec_39
        ld      a,[cursor]
        cp      $3A
        jr      nz,not_dec_3A
        ld      a,[val_FF26]
        and     $7F
        ld      b,a
        ld      a,[val_FF26]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF26],a

not_dec_3A
        ld      a,[cursor]
        cp      $3B
        jr      nz,not_dec_3B
        ld      a,[val_FF26]
        and     $80
        ld      b,a
        ld      a,[val_FF26]
        and     $7F
        sub     $01
        and     $7F
        add     b
        ld      [val_FF26],a

not_dec_3B
        ld      a,[cursor]
        cp      $3C
        jr      nz,not_dec_3C
        ld      a,[val_FF30]
        and     $0F
        ld      b,a
        ld      a,[val_FF30]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF30],a

not_dec_3C
        ld      a,[cursor]
        cp      $3D
        jr      nz,not_dec_3D
        ld      a,[val_FF30]
        and     $F0
        ld      b,a
        ld      a,[val_FF30]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF30],a

not_dec_3D
        ld      a,[cursor]
        cp      $3E
        jr      nz,not_dec_3E
        ld      a,[val_FF31]
        and     $0F
        ld      b,a
        ld      a,[val_FF31]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF31],a

not_dec_3E
        ld      a,[cursor]
        cp      $3F
        jr      nz,not_dec_3F
        ld      a,[val_FF31]
        and     $F0
        ld      b,a
        ld      a,[val_FF31]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF31],a

not_dec_3F
        ld      a,[cursor]
        cp      $40
        jr      nz,not_dec_40
        ld      a,[val_FF32]
        and     $0F
        ld      b,a
        ld      a,[val_FF32]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF32],a

not_dec_40
        ld      a,[cursor]
        cp      $41
        jr      nz,not_dec_41
        ld      a,[val_FF32]
        and     $F0
        ld      b,a
        ld      a,[val_FF32]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF32],a

not_dec_41
        ld      a,[cursor]
        cp      $42
        jr      nz,not_dec_42
        ld      a,[val_FF33]
        and     $0F
        ld      b,a
        ld      a,[val_FF33]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF33],a

not_dec_42
        ld      a,[cursor]
        cp      $43
        jr      nz,not_dec_43
        ld      a,[val_FF33]
        and     $F0
        ld      b,a
        ld      a,[val_FF33]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF33],a

not_dec_43
        ld      a,[cursor]
        cp      $44
        jr      nz,not_dec_44
        ld      a,[val_FF34]
        and     $0F
        ld      b,a
        ld      a,[val_FF34]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF34],a

not_dec_44
        ld      a,[cursor]
        cp      $45
        jr      nz,not_dec_45
        ld      a,[val_FF34]
        and     $F0
        ld      b,a
        ld      a,[val_FF34]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF34],a

not_dec_45
        ld      a,[cursor]
        cp      $46
        jr      nz,not_dec_46
        ld      a,[val_FF35]
        and     $0F
        ld      b,a
        ld      a,[val_FF35]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF35],a

not_dec_46
        ld      a,[cursor]
        cp      $47
        jr      nz,not_dec_47
        ld      a,[val_FF35]
        and     $F0
        ld      b,a
        ld      a,[val_FF35]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF35],a

not_dec_47
        ld      a,[cursor]
        cp      $48
        jr      nz,not_dec_48
        ld      a,[val_FF36]
        and     $0F
        ld      b,a
        ld      a,[val_FF36]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF36],a

not_dec_48
        ld      a,[cursor]
        cp      $49
        jr      nz,not_dec_49
        ld      a,[val_FF36]
        and     $F0
        ld      b,a
        ld      a,[val_FF36]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF36],a

not_dec_49
        ld      a,[cursor]
        cp      $4A
        jr      nz,not_dec_4A
        ld      a,[val_FF37]
        and     $0F
        ld      b,a
        ld      a,[val_FF37]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF37],a

not_dec_4A
        ld      a,[cursor]
        cp      $4B
        jr      nz,not_dec_4B
        ld      a,[val_FF37]
        and     $F0
        ld      b,a
        ld      a,[val_FF37]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF37],a

not_dec_4B
        ld      a,[cursor]
        cp      $4C
        jr      nz,not_dec_4C
        ld      a,[val_FF38]
        and     $0F
        ld      b,a
        ld      a,[val_FF38]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF38],a

not_dec_4C
        ld      a,[cursor]
        cp      $4D
        jr      nz,not_dec_4D
        ld      a,[val_FF38]
        and     $F0
        ld      b,a
        ld      a,[val_FF38]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF38],a

not_dec_4D
        ld      a,[cursor]
        cp      $4E
        jr      nz,not_dec_4E
        ld      a,[val_FF39]
        and     $0F
        ld      b,a
        ld      a,[val_FF39]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF39],a

not_dec_4E
        ld      a,[cursor]
        cp      $4F
        jr      nz,not_dec_4F
        ld      a,[val_FF39]
        and     $F0
        ld      b,a
        ld      a,[val_FF39]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF39],a

not_dec_4F
        ld      a,[cursor]
        cp      $50
        jr      nz,not_dec_50
        ld      a,[val_FF3A]
        and     $0F
        ld      b,a
        ld      a,[val_FF3A]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF3A],a

not_dec_50
        ld      a,[cursor]
        cp      $51
        jr      nz,not_dec_51
        ld      a,[val_FF3A]
        and     $F0
        ld      b,a
        ld      a,[val_FF3A]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF3A],a

not_dec_51
        ld      a,[cursor]
        cp      $52
        jr      nz,not_dec_52
        ld      a,[val_FF3B]
        and     $0F
        ld      b,a
        ld      a,[val_FF3B]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF3B],a

not_dec_52
        ld      a,[cursor]
        cp      $53
        jr      nz,not_dec_53
        ld      a,[val_FF3B]
        and     $F0
        ld      b,a
        ld      a,[val_FF3B]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF3B],a

not_dec_53
        ld      a,[cursor]
        cp      $54
        jr      nz,not_dec_54
        ld      a,[val_FF3C]
        and     $0F
        ld      b,a
        ld      a,[val_FF3C]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF3C],a

not_dec_54
        ld      a,[cursor]
        cp      $55
        jr      nz,not_dec_55
        ld      a,[val_FF3C]
        and     $F0
        ld      b,a
        ld      a,[val_FF3C]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF3C],a

not_dec_55
        ld      a,[cursor]
        cp      $56
        jr      nz,not_dec_56
        ld      a,[val_FF3D]
        and     $0F
        ld      b,a
        ld      a,[val_FF3D]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF3D],a

not_dec_56
        ld      a,[cursor]
        cp      $57
        jr      nz,not_dec_57
        ld      a,[val_FF3D]
        and     $F0
        ld      b,a
        ld      a,[val_FF3D]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF3D],a

not_dec_57
        ld      a,[cursor]
        cp      $58
        jr      nz,not_dec_58
        ld      a,[val_FF3E]
        and     $0F
        ld      b,a
        ld      a,[val_FF3E]
        and     $F0  
        sub     $10
        and     $F0
        add     b
        ld      [val_FF3E],a

not_dec_58
        ld      a,[cursor]
        cp      $59
        jr      nz,not_dec_59
        ld      a,[val_FF3E]
        and     $F0
        ld      b,a
        ld      a,[val_FF3E]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF3E],a

not_dec_59
        ld      a,[cursor]
        cp      $5A
        jr      nz,not_dec_5A
        ld      a,[val_FF3F]
        and     $0F
        ld      b,a
        ld      a,[val_FF3F]
        and     $F0
        sub     $10
        and     $F0
        add     b
        ld      [val_FF3F],a

not_dec_5A
        ld      a,[cursor]
        cp      $5B
        jr      nz,not_dec_5B
        ld      a,[val_FF3F]
        and     $F0
        ld      b,a
        ld      a,[val_FF3F]
        and     $0F
        sub     $01
        and     $0F
        add     b
        ld      [val_FF3F],a

not_dec_5B
        call    update_NR

not_DOWN
        ld      a,[button_debounced]
        bit     PADB_UP,a
        jp      z,not_UP

        res     PADB_UP,a
        ld      [button_debounced],a

        ld      a,[cursor]
        cp      $00
        jr      nz,not_inc_00
        ld      a,[val_FF10]
        and     $7F
        ld      b,a
        ld      a,[val_FF10]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF10],a

not_inc_00
        ld      a,[cursor]
        cp      $01
        jr      nz,not_inc_01
        ld      a,[val_FF10]
        and     $8F
        ld      b,a
        ld      a,[val_FF10]
        and     $70
        add     $10
        and     $70
        add     b
        ld      [val_FF10],a

not_inc_01
        ld      a,[cursor]
        cp      $02
        jr      nz,not_inc_02
        ld      a,[val_FF10]
        and     $F7
        ld      b,a
        ld      a,[val_FF10]
        and     $08
        add     $08
        and     $08
        add     b
        ld      [val_FF10],a

not_inc_02
        ld      a,[cursor]
        cp      $03
        jr      nz,not_inc_03
        ld      a,[val_FF10]
        and     $F8
        ld      b,a
        ld      a,[val_FF10]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF10],a

not_inc_03
        ld      a,[cursor]
        cp      $04
        jr      nz,not_inc_04
        ld      a,[val_FF11]
        and     $3F
        ld      b,a
        ld      a,[val_FF11]
        and     $C0
        add     $40
        and     $C0
        add     b
        ld      [val_FF11],a

not_inc_04
        ld      a,[cursor]
        cp      $05
        jr      nz,not_inc_05
        ld      a,[val_FF11]
        and     $C0
        ld      b,a
        ld      a,[val_FF11]
        and     $3F
        add     $01
        and     $3F
        add     b
        ld      [val_FF11],a

not_inc_05
        ld      a,[cursor]
        cp      $06
        jr      nz,not_inc_06
        ld      a,[val_FF12]
        and     $0F
        ld      b,a
        ld      a,[val_FF12]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF12],a

not_inc_06
        ld      a,[cursor]
        cp      $07
        jr      nz,not_inc_07
        ld      a,[val_FF12]
        and     $F7
        ld      b,a
        ld      a,[val_FF12]
        and     $08
        add     $08
        and     $08
        add     b
        ld      [val_FF12],a

not_inc_07
        ld      a,[cursor]
        cp      $08
        jr      nz,not_inc_08
        ld      a,[val_FF12]
        and     $F8
        ld      b,a
        ld      a,[val_FF12]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF12],a

not_inc_08
        ld      a,[cursor]
        cp      $09
        jr      nz,not_inc_09
        ld      a,[val_FF13]
        add     $01
        ld      [val_FF13],a

not_inc_09
        ld      a,[cursor]
        cp      $0A
        jr      nz,not_inc_0A
        ld      a,[val_FF14]
        and     $7F
        ld      b,a
        ld      a,[val_FF14]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF14],a

not_inc_0A
        ld      a,[cursor]
        cp      $0B
        jr      nz,not_inc_0B
        ld      a,[val_FF14]
        and     $BF
        ld      b,a
        ld      a,[val_FF14]
        and     $40
        add     $40
        and     $40
        add     b
        ld      [val_FF14],a

not_inc_0B
        ld      a,[cursor]
        cp      $0C
        jr      nz,not_inc_0C
        ld      a,[val_FF14]
        and     $C7
        ld      b,a
        ld      a,[val_FF14]
        and     $38
        add     $08
        and     $38
        add     b
        ld      [val_FF14],a

not_inc_0C
        ld      a,[cursor]
        cp      $0D
        jr      nz,not_inc_0D
        ld      a,[val_FF14]
        and     $F8
        ld      b,a
        ld      a,[val_FF14]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF14],a

not_inc_0D
        ld      a,[cursor]
        cp      $0E
        jr      nz,not_inc_0E
        ld      a,[val_FF16]
        and     $3F
        ld      b,a
        ld      a,[val_FF16]
        and     $C0
        add     $40
        and     $C0
        add     b
        ld      [val_FF16],a

not_inc_0E
        ld      a,[cursor]
        cp      $0F
        jr      nz,not_inc_0F
        ld      a,[val_FF16]
        and     $C0
        ld      b,a
        ld      a,[val_FF16]
        and     $3F
        add     $01
        and     $3F
        add     b
        ld      [val_FF16],a

not_inc_0F
        ld      a,[cursor]
        cp      $10
        jr      nz,not_inc_10
        ld      a,[val_FF17]
        and     $0F
        ld      b,a
        ld      a,[val_FF17]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF17],a

not_inc_10
        ld      a,[cursor]
        cp      $11
        jr      nz,not_inc_11
        ld      a,[val_FF17]
        and     $F7
        ld      b,a
        ld      a,[val_FF17]
        and     $08
        add     $08
        and     $08
        add     b
        ld      [val_FF17],a

not_inc_11
        ld      a,[cursor]
        cp      $12
        jr      nz,not_inc_12
        ld      a,[val_FF17]
        and     $F8
        ld      b,a
        ld      a,[val_FF17]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF17],a

not_inc_12
        ld      a,[cursor]
        cp      $13
        jr      nz,not_inc_13
        ld      a,[val_FF18]
        add     $01
        ld      [val_FF18],a

not_inc_13
        ld      a,[cursor]
        cp      $14
        jr      nz,not_inc_14
        ld      a,[val_FF19]
        and     $7F
        ld      b,a
        ld      a,[val_FF19]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF19],a

not_inc_14
        ld      a,[cursor]
        cp      $15
        jr      nz,not_inc_15
        ld      a,[val_FF19]
        and     $BF
        ld      b,a
        ld      a,[val_FF19]
        and     $40
        add     $40
        and     $40
        add     b
        ld      [val_FF19],a

not_inc_15
        ld      a,[cursor]
        cp      $16
        jr      nz,not_inc_16
        ld      a,[val_FF19]
        and     $C7
        ld      b,a
        ld      a,[val_FF19]
        and     $38
        add     $08
        and     $38
        add     b
        ld      [val_FF19],a

not_inc_16
        ld      a,[cursor]
        cp      $17
        jr      nz,not_inc_17
        ld      a,[val_FF19]
        and     $F8
        ld      b,a
        ld      a,[val_FF19]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF19],a

not_inc_17
        ld      a,[cursor]
        cp      $18
        jr      nz,not_inc_18
        ld      a,[val_FF1A]
        and     $7F
        ld      b,a
        ld      a,[val_FF1A]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF1A],a

not_inc_18
        ld      a,[cursor]
        cp      $19
        jr      nz,not_inc_19
        ld      a,[val_FF1A]
        and     $80
        ld      b,a
        ld      a,[val_FF1A]
        and     $7F
        add     $01
        and     $7F
        add     b
        ld      [val_FF1A],a

not_inc_19
        ld      a,[cursor]
        cp      $1A
        jr      nz,not_inc_1A
        ld      a,[val_FF1B]
        add     $01
        ld      [val_FF1B],a

not_inc_1A
        ld      a,[cursor]
        cp      $1B
        jr      nz,not_inc_1B
        ld      a,[val_FF1C]
        and     $7F
        ld      b,a
        ld      a,[val_FF1C]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF1C],a

not_inc_1B
        ld      a,[cursor]
        cp      $1C
        jr      nz,not_inc_1C
        ld      a,[val_FF1C]
        and     $9F
        ld      b,a
        ld      a,[val_FF1C]
        and     $60
        add     $20
        and     $60
        add     b
        ld      [val_FF1C],a

not_inc_1C
        ld      a,[cursor]
        cp      $1D
        jr      nz,not_inc_1D
        ld      a,[val_FF1C]
        and     $E0
        ld      b,a
        ld      a,[val_FF1C]
        and     $1F
        add     $01
        and     $1F
        add     b
        ld      [val_FF1C],a

not_inc_1D
        ld      a,[cursor]
        cp      $1E
        jr      nz,not_inc_1E
        ld      a,[val_FF1D]
        add     $01
        ld      [val_FF1D],a

not_inc_1E
        ld      a,[cursor]
        cp      $1F
        jr      nz,not_inc_1F
        ld      a,[val_FF1E]
        and     $7F
        ld      b,a
        ld      a,[val_FF1E]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF1E],a

not_inc_1F
        ld      a,[cursor]
        cp      $20
        jr      nz,not_inc_20
        ld      a,[val_FF1E]
        and     $BF
        ld      b,a
        ld      a,[val_FF1E]
        and     $40
        add     $40
        and     $40
        add     b
        ld      [val_FF1E],a

not_inc_20
        ld      a,[cursor]
        cp      $21
        jr      nz,not_inc_21
        ld      a,[val_FF1E]
        and     $C7
        ld      b,a
        ld      a,[val_FF1E]
        and     $38
        add     $08
        and     $38
        add     b
        ld      [val_FF1E],a

not_inc_21
        ld      a,[cursor]
        cp      $22
        jr      nz,not_inc_22
        ld      a,[val_FF1E]
        and     $F8
        ld      b,a
        ld      a,[val_FF1E]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF1E],a

not_inc_22
        ld      a,[cursor]
        cp      $23
        jr      nz,not_inc_23
        ld      a,[val_FF20]
        and     $3F
        ld      b,a
        ld      a,[val_FF20]
        and     $C0
        add     $40
        and     $C0
        add     b
        ld      [val_FF20],a

not_inc_23
        ld      a,[cursor]
        cp      $24
        jr      nz,not_inc_24
        ld      a,[val_FF20]
        and     $C0
        ld      b,a
        ld      a,[val_FF20]
        and     $3F
        add     $01
        and     $3F
        add     b
        ld      [val_FF20],a

not_inc_24
        ld      a,[cursor]
        cp      $25
        jr      nz,not_inc_25
        ld      a,[val_FF21]
        and     $0F
        ld      b,a
        ld      a,[val_FF21]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF21],a

not_inc_25
        ld      a,[cursor]
        cp      $26
        jr      nz,not_inc_26
        ld      a,[val_FF21]
        and     $F7
        ld      b,a
        ld      a,[val_FF21]
        and     $08
        add     $08
        and     $08
        add     b
        ld      [val_FF21],a

not_inc_26
        ld      a,[cursor]
        cp      $27
        jr      nz,not_inc_27
        ld      a,[val_FF21]
        and     $F8
        ld      b,a
        ld      a,[val_FF21]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF21],a

not_inc_27
        ld      a,[cursor]
        cp      $28
        jr      nz,not_inc_28
        ld      a,[val_FF22]
        and     $0F
        ld      b,a
        ld      a,[val_FF22]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF22],a

not_inc_28
        ld      a,[cursor]
        cp      $29
        jr      nz,not_inc_29
        ld      a,[val_FF22]
        and     $F7
        ld      b,a
        ld      a,[val_FF22]
        and     $08
        add     $08
        and     $08
        add     b
        ld      [val_FF22],a

not_inc_29
        ld      a,[cursor]
        cp      $2A
        jr      nz,not_inc_2A
        ld      a,[val_FF22]
        and     $F8
        ld      b,a
        ld      a,[val_FF22]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF22],a

not_inc_2A
        ld      a,[cursor]
        cp      $2B
        jr      nz,not_inc_2B
        ld      a,[val_FF23]
        and     $7F
        ld      b,a
        ld      a,[val_FF23]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF23],a

not_inc_2B
        ld      a,[cursor]
        cp      $2C
        jr      nz,not_inc_2C
        ld      a,[val_FF23]
        and     $BF
        ld      b,a
        ld      a,[val_FF23]
        and     $40
        add     $40
        and     $40
        add     b
        ld      [val_FF23],a

not_inc_2C
        ld      a,[cursor]
        cp      $2D
        jr      nz,not_inc_2D
        ld      a,[val_FF23]
        and     $C0
        ld      b,a
        ld      a,[val_FF23]
        and     $3F
        add     $01
        and     $3F
        add     b
        ld      [val_FF23],a

not_inc_2D
        ld      a,[cursor]
        cp      $2E
        jr      nz,not_inc_2E
        ld      a,[val_FF24]
        and     $7F
        ld      b,a
        ld      a,[val_FF24]
        and     $80  
        add     $80
        and     $80
        add     b
        ld      [val_FF24],a

not_inc_2E
        ld      a,[cursor]
        cp      $2F
        jr      nz,not_inc_2F
        ld      a,[val_FF24]
        and     $8F
        ld      b,a
        ld      a,[val_FF24]
        and     $70
        add     $10
        and     $70
        add     b
        ld      [val_FF24],a

not_inc_2F
        ld      a,[cursor]
        cp      $30
        jr      nz,not_inc_30
        ld      a,[val_FF24]
        and     $F7
        ld      b,a
        ld      a,[val_FF24]
        and     $08
        add     $08
        and     $08
        add     b
        ld      [val_FF24],a

not_inc_30
        ld      a,[cursor]
        cp      $31
        jr      nz,not_inc_31
        ld      a,[val_FF24]
        and     $F8
        ld      b,a
        ld      a,[val_FF24]
        and     $07
        add     $01
        and     $07
        add     b
        ld      [val_FF24],a

not_inc_31
        ld      a,[cursor]
        cp      $32
        jr      nz,not_inc_32
        ld      a,[val_FF25]
        and     $7F
        ld      b,a
        ld      a,[val_FF25]
        and     $80
        sub     $80
        and     $80
        add     b
        ld      [val_FF25],a

not_inc_32
        ld      a,[cursor]
        cp      $33
        jr      nz,not_inc_33
        ld      a,[val_FF25]
        and     $BF
        ld      b,a
        ld      a,[val_FF25]
        and     $40
        sub     $40
        and     $40
        add     b
        ld      [val_FF25],a

not_inc_33
        ld      a,[cursor]
        cp      $34
        jr      nz,not_inc_34
        ld      a,[val_FF25]
        and     $DF
        ld      b,a
        ld      a,[val_FF25]
        and     $20
        add     $20
        and     $20
        add     b
        ld      [val_FF25],a

not_inc_34
        ld      a,[cursor]
        cp      $35
        jr      nz,not_inc_35
        ld      a,[val_FF25]
        and     $EF
        ld      b,a
        ld      a,[val_FF25]
        and     $10
        add     $10
        and     $10
        add     b
        ld      [val_FF25],a

not_inc_35
        ld      a,[cursor]
        cp      $36
        jr      nz,not_inc_36
        ld      a,[val_FF25]
        and     $F7
        ld      b,a
        ld      a,[val_FF25]
        and     $08
        sub     $08
        and     $08
        add     b
        ld      [val_FF25],a

not_inc_36
        ld      a,[cursor]
        cp      $37
        jr      nz,not_inc_37
        ld      a,[val_FF25]
        and     $FB
        ld      b,a
        ld      a,[val_FF25]
        and     $04
        add     $04
        and     $04
        add     b
        ld      [val_FF25],a

not_inc_37
        ld      a,[cursor]
        cp      $38
        jr      nz,not_inc_38
        ld      a,[val_FF25]
        and     $FD
        ld      b,a
        ld      a,[val_FF25]
        and     $02
        add     $02
        and     $02
        add     b
        ld      [val_FF25],a

not_inc_38
        ld      a,[cursor]
        cp      $39
        jr      nz,not_inc_39
        ld      a,[val_FF25]
        and     $FE
        ld      b,a
        ld      a,[val_FF25]
        and     $01
        add     $01
        and     $01
        add     b
        ld      [val_FF25],a

not_inc_39
        ld      a,[cursor]
        cp      $3A
        jr      nz,not_inc_3A
        ld      a,[val_FF26]
        and     $7F
        ld      b,a
        ld      a,[val_FF26]
        and     $80
        add     $80
        and     $80
        add     b
        ld      [val_FF26],a

not_inc_3A
        ld      a,[cursor]
        cp      $3B
        jr      nz,not_inc_3B
        ld      a,[val_FF26]
        and     $80
        ld      b,a
        ld      a,[val_FF26]
        and     $7F
        add     $01
        and     $7F
        add     b
        ld      [val_FF26],a

not_inc_3B
        ld      a,[cursor]
        cp      $3C
        jr      nz,not_inc_3C
        ld      a,[val_FF30]
        and     $0F
        ld      b,a
        ld      a,[val_FF30]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF30],a

not_inc_3C
        ld      a,[cursor]
        cp      $3D
        jr      nz,not_inc_3D
        ld      a,[val_FF30]
        and     $F0
        ld      b,a
        ld      a,[val_FF30]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF30],a

not_inc_3D
        ld      a,[cursor]
        cp      $3E
        jr      nz,not_inc_3E
        ld      a,[val_FF31]
        and     $0F
        ld      b,a
        ld      a,[val_FF31]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF31],a

not_inc_3E
        ld      a,[cursor]
        cp      $3F
        jr      nz,not_inc_3F
        ld      a,[val_FF31]
        and     $F0
        ld      b,a
        ld      a,[val_FF31]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF31],a

not_inc_3F
        ld      a,[cursor]
        cp      $40
        jr      nz,not_inc_40
        ld      a,[val_FF32]
        and     $0F
        ld      b,a
        ld      a,[val_FF32]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF32],a

not_inc_40
        ld      a,[cursor]
        cp      $41
        jr      nz,not_inc_41
        ld      a,[val_FF32]
        and     $F0
        ld      b,a
        ld      a,[val_FF32]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF32],a

not_inc_41
        ld      a,[cursor]
        cp      $42
        jr      nz,not_inc_42
        ld      a,[val_FF33]
        and     $0F
        ld      b,a
        ld      a,[val_FF33]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF33],a

not_inc_42
        ld      a,[cursor]
        cp      $43
        jr      nz,not_inc_43
        ld      a,[val_FF33]
        and     $F0
        ld      b,a
        ld      a,[val_FF33]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF33],a

not_inc_43
        ld      a,[cursor]
        cp      $44
        jr      nz,not_inc_44
        ld      a,[val_FF34]
        and     $0F
        ld      b,a
        ld      a,[val_FF34]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF34],a

not_inc_44
        ld      a,[cursor]
        cp      $45
        jr      nz,not_inc_45
        ld      a,[val_FF34]
        and     $F0
        ld      b,a
        ld      a,[val_FF34]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF34],a

not_inc_45
        ld      a,[cursor]
        cp      $46
        jr      nz,not_inc_46
        ld      a,[val_FF35]
        and     $0F
        ld      b,a
        ld      a,[val_FF35]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF35],a

not_inc_46
        ld      a,[cursor]
        cp      $47
        jr      nz,not_inc_47
        ld      a,[val_FF35]
        and     $F0
        ld      b,a
        ld      a,[val_FF35]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF35],a

not_inc_47
        ld      a,[cursor]
        cp      $48
        jr      nz,not_inc_48
        ld      a,[val_FF36]
        and     $0F
        ld      b,a
        ld      a,[val_FF36]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF36],a

not_inc_48
        ld      a,[cursor]
        cp      $49
        jr      nz,not_inc_49
        ld      a,[val_FF36]
        and     $F0
        ld      b,a
        ld      a,[val_FF36]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF36],a

not_inc_49
        ld      a,[cursor]
        cp      $4A
        jr      nz,not_inc_4A
        ld      a,[val_FF37]
        and     $0F
        ld      b,a
        ld      a,[val_FF37]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF37],a

not_inc_4A
        ld      a,[cursor]
        cp      $4B
        jr      nz,not_inc_4B
        ld      a,[val_FF37]
        and     $F0
        ld      b,a
        ld      a,[val_FF37]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF37],a

not_inc_4B
        ld      a,[cursor]
        cp      $4C
        jr      nz,not_inc_4C
        ld      a,[val_FF38]
        and     $0F
        ld      b,a
        ld      a,[val_FF38]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF38],a

not_inc_4C
        ld      a,[cursor]
        cp      $4D
        jr      nz,not_inc_4D
        ld      a,[val_FF38]
        and     $F0
        ld      b,a
        ld      a,[val_FF38]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF38],a

not_inc_4D
        ld      a,[cursor]
        cp      $4E
        jr      nz,not_inc_4E
        ld      a,[val_FF39]
        and     $0F
        ld      b,a
        ld      a,[val_FF39]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF39],a

not_inc_4E
        ld      a,[cursor]
        cp      $4F
        jr      nz,not_inc_4F
        ld      a,[val_FF39]
        and     $F0
        ld      b,a
        ld      a,[val_FF39]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF39],a

not_inc_4F
        ld      a,[cursor]
        cp      $50
        jr      nz,not_inc_50
        ld      a,[val_FF3A]
        and     $0F
        ld      b,a
        ld      a,[val_FF3A]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF3A],a

not_inc_50
        ld      a,[cursor]
        cp      $51
        jr      nz,not_inc_51
        ld      a,[val_FF3A]
        and     $F0
        ld      b,a
        ld      a,[val_FF3A]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF3A],a

not_inc_51
        ld      a,[cursor]
        cp      $52
        jr      nz,not_inc_52
        ld      a,[val_FF3B]
        and     $0F
        ld      b,a
        ld      a,[val_FF3B]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF3B],a

not_inc_52
        ld      a,[cursor]
        cp      $53
        jr      nz,not_inc_53
        ld      a,[val_FF3B]
        and     $F0
        ld      b,a
        ld      a,[val_FF3B]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF3B],a

not_inc_53
        ld      a,[cursor]
        cp      $54
        jr      nz,not_inc_54
        ld      a,[val_FF3C]
        and     $0F
        ld      b,a
        ld      a,[val_FF3C]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF3C],a

not_inc_54
        ld      a,[cursor]
        cp      $55
        jr      nz,not_inc_55
        ld      a,[val_FF3C]
        and     $F0
        ld      b,a
        ld      a,[val_FF3C]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF3C],a

not_inc_55
        ld      a,[cursor]
        cp      $56
        jr      nz,not_inc_56
        ld      a,[val_FF3D]
        and     $0F
        ld      b,a
        ld      a,[val_FF3D]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF3D],a

not_inc_56
        ld      a,[cursor]
        cp      $57
        jr      nz,not_inc_57
        ld      a,[val_FF3D]
        and     $F0
        ld      b,a
        ld      a,[val_FF3D]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF3D],a

not_inc_57
        ld      a,[cursor]
        cp      $58
        jr      nz,not_inc_58
        ld      a,[val_FF3E]
        and     $0F
        ld      b,a
        ld      a,[val_FF3E]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF3E],a

not_inc_58
        ld      a,[cursor]
        cp      $59
        jr      nz,not_inc_59
        ld      a,[val_FF3E]
        and     $F0
        ld      b,a
        ld      a,[val_FF3E]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF3E],a

not_inc_59
        ld      a,[cursor]
        cp      $5A
        jr      nz,not_inc_5A
        ld      a,[val_FF3F]
        and     $0F
        ld      b,a
        ld      a,[val_FF3F]
        and     $F0
        add     $10
        and     $F0
        add     b
        ld      [val_FF3F],a

not_inc_5A
        ld      a,[cursor]
        cp      $5B
        jr      nz,not_inc_5B
        ld      a,[val_FF3F]
        and     $F0
        ld      b,a
        ld      a,[val_FF3F]
        and     $0F
        add     $01
        and     $0F
        add     b
        ld      [val_FF3F],a

not_inc_5B
        call    update_NR

not_UP
        ld      a,[button_debounced]
        bit     PADB_A,a
        jp      z,not_A

        res     PADB_A,a
        ld      [button_debounced],a

        call    update_NR

not_A
        ld      a,[button_debounced]
        bit     PADB_B,a
        jp      z,not_B

        res     PADB_B,a
        ld      [button_debounced],a

        ld      a,[val_FF14]
        or      $80
        ld      [val_FF14],a
        ld      a,[val_FF19]
        or      $80
        ld      [val_FF19],a
        ld      a,[val_FF1A]
        or      $80
        ld      [val_FF1A],a
        ld      a,[val_FF1E]
        or      $80
        ld      [val_FF1E],a
        ld      a,[val_FF23]
        or      $80
        ld      [val_FF23],a
        ld      a,$FF
        ld      [val_FF25],a

        call    update_NR

not_B
        ret

table_jmp
        sla     a
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      a,[hl]
        ld      e,a
        inc     hl
        ld      a,[hl]
        ld      d,a
        push    de
        ret

write_to_screen
        ld      hl,pos_arrows
        ld      a,$1A
        ld      [hl-],a
        add     $01
        ld      [hl-],a

        ld      a,[NR_update]
        ld      hl,pos_table
        jp      table_jmp

show_pos0
        ld      a,[FirstLine]
        sla     a
        ld      hl,text_FirstLine
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      de,pos_0-11
        ld      bc,$02
        call    mem_Copy

        ld      a,[FirstLine]
        ld      hl,val_FF10
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        push    hl
        ld      a,[hl]
        and     $0F
        ld      hl,pos_0+4
        call    show_it

        pop     hl
        ld      a,[hl]
        swap    a
        and     $0F
        ld      hl,pos_0+3
        call    show_it

        ld      a,[FirstLine]
        ld      hl,show_table
        jp      table_jmp

show_pos1
        ld      a,[FirstLine]
        add     $01
        cp      last_line+1
        jr      c,no_lbl_1

        sub     last_line+1

no_lbl_1
        sla     a
        ld      hl,text_FirstLine
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      de,pos_1-11
        ld      bc,$02
        call    mem_Copy

        ld      a,[FirstLine]
        add     $01
        cp      last_line+1
        jr      c,no_val_1

        sub     last_line+1

no_val_1
        ld      hl,val_FF10
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        push    hl
        ld      a,[hl]
        and     $0F
        ld      hl,pos_1+4
        call    show_it

        pop     hl
        ld      a,[hl]
        swap    a
        and     $0F
        ld      hl,pos_1+3
        call    show_it

        ld      a,[FirstLine]
        add     $01
        cp      last_line+1
        jr      c,no_adj_1

        sub     last_line+1

no_adj_1
        ld      hl,show_table
        jp      table_jmp

show_pos2
        ld      a,[FirstLine]
        add     $02
        cp      last_line+1
        jr      c,no_lbl_2

        sub     last_line+1

no_lbl_2
        sla     a
        ld      hl,text_FirstLine
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      de,pos_2-11
        ld      bc,$02
        call    mem_Copy

        ld      a,[FirstLine]
        add     $02
        cp      last_line+1
        jr      c,no_val_2

        sub     last_line+1

no_val_2
        ld      hl,val_FF10
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        push    hl
        ld      a,[hl]
        and     $0F
        ld      hl,pos_2+4
        call    show_it

        pop     hl
        ld      a,[hl]
        swap    a
        and     $0F
        ld      hl,pos_2+3
        call    show_it

        ld      a,[FirstLine]
        add     $02
        cp      last_line+1
        jr      c,no_adj_2

        sub     last_line+1

no_adj_2
        ld      hl,show_table
        jp      table_jmp

show_pos3
        ld      a,[FirstLine]
        add     $03
        cp      last_line+1
        jr      c,no_lbl_3

        sub     last_line+1

no_lbl_3
        sla     a
        ld      hl,text_FirstLine
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      de,pos_3-11
        ld      bc,$02
        call    mem_Copy

        ld      a,[FirstLine]
        add     $03
        cp      last_line+1
        jr      c,no_val_3

        sub     last_line+1

no_val_3
        ld      hl,val_FF10
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        push    hl
        ld      a,[hl]
        and     $0F
        ld      hl,pos_3+4
        call    show_it

        pop     hl
        ld      a,[hl]
        swap    a
        and     $0F
        ld      hl,pos_3+3
        call    show_it

        ld      a,[FirstLine]
        add     $03
        cp      last_line+1
        jr      c,no_adj_3

        sub     last_line+1

no_adj_3
        ld      hl,show_table
        jp      table_jmp

show_pos4
        ld      a,[FirstLine]
        add     $04
        cp      last_line+1
        jr      c,no_lbl_4

        sub     last_line+1

no_lbl_4
        sla     a
        ld      hl,text_FirstLine
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      de,pos_4-11
        ld      bc,$02
        call    mem_Copy

        ld      a,[FirstLine]
        add     $04
        cp      last_line+1
        jr      c,no_val_4

        sub     last_line+1

no_val_4
        ld      hl,val_FF10
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        push    hl
        ld      a,[hl]
        and     $0F
        ld      hl,pos_4+4
        call    show_it

        pop     hl
        ld      a,[hl]
        swap    a
        and     $0F
        ld      hl,pos_4+3
        call    show_it

        ld      a,[FirstLine]
        add     $04
        cp      last_line+1
        jr      c,no_adj_4

        sub     last_line+1

no_adj_4
        ld      hl,show_table
        jp      table_jmp

show_pos5
        ld      a,[FirstLine]
        add     $05
        cp      last_line+1
        jr      c,no_lbl_5

        sub     last_line+1

no_lbl_5
        sla     a
        ld      hl,text_FirstLine
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      de,pos_5-11
        ld      bc,$02
        call    mem_Copy

        ld      a,[FirstLine]
        add     $05
        cp      last_line+1
        jr      c,no_val_5

        sub     last_line+1

no_val_5
        ld      hl,val_FF10
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        push    hl
        ld      a,[hl]
        and     $0F
        ld      hl,pos_5+4
        call    show_it

        pop     hl
        ld      a,[hl]
        swap    a
        and     $0F
        ld      hl,pos_5+3
        call    show_it

        ld      a,[FirstLine]
        add     $05
        cp      last_line+1
        jr      c,no_adj_5

        sub     last_line+1

no_adj_5
        ld      hl,show_table
        jp      table_jmp

show_pos6
        ld      a,[FirstLine]
        add     $06
        cp      last_line+1
        jr      c,no_lbl_6

        sub     last_line+1

no_lbl_6
        sla     a
        ld      hl,text_FirstLine
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      de,pos_6-11
        ld      bc,$02
        call    mem_Copy

        ld      a,[FirstLine]
        add     $06
        cp      last_line+1
        jr      c,no_val_6

        sub     last_line+1

no_val_6
        ld      hl,val_FF10
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        push    hl
        ld      a,[hl]
        and     $0F
        ld      hl,pos_6+4
        call    show_it

        pop     hl
        ld      a,[hl]
        swap    a
        and     $0F
        ld      hl,pos_6+3
        call    show_it

        ld      a,[FirstLine]
        add     $06
        cp      last_line+1
        jr      c,no_adj_6

        sub     last_line+1

no_adj_6
        ld      hl,show_table
        jp      table_jmp

display_text
        ld      de,pos_text
        ld      bc,18
        call    mem_Copy

        ret

adjust_line
        ld      a,[NR_update]
        ld      hl,display_table
        sla     a
        add     a,l
        ld      l,a
        ld      a,h
        adc     $00
        ld      h,a
        ld      a,[hl]
        ld      e,a
        inc     hl
        ld      a,[hl]
        ld      d,a
        push    de
        pop     hl
        ret

show_text
        ld      a,[TextH]
        ld      h,a
        ld      a,[TextL]
        ld      l,a
        call    display_text
        ret

show_NR10
        ld      a,[cursor]
        cp      $03
        jr      nz,no_cursor_03

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_03

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_03
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_03

no_cursor_03
        call    adjust_line
        ld      a,[val_FF10]
        and     $01
        call    show_it

        ld      a,[val_FF10]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF10]
        and     $04
        srl     a
        srl     a
        call    show_it

after_03
        ld      a,[cursor]
        cp      $02
        jr      nz,no_cursor_02

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_02

        call    adjust_line
        dec     hl
        dec     hl
        dec     hl
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_02
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_02

no_cursor_02
        ld      a,[val_FF10]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_02
        ld      a,[cursor]
        cp      $01
        jr      nz,no_cursor_01

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_01

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_01
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_01

no_cursor_01
        ld      a,[val_FF10]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF10]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF10]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_01
        ld      a,[cursor]
        cp      $00
        jr      nz,no_cursor_00

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_00

        call    adjust_line
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        dec     hl
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_1x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_00

no_cursor_00
        ld      a,[val_FF10]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_00
        ret

show_NR11
        ld      a,[cursor]
        cp      $05
        jr      nz,no_cursor_05

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_05

        call    adjust_line
        ld      a,$20
        REPT    6
        ld      [hl-],a
        ENDR

        ld      hl,text_05
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_05

no_cursor_05
        call    adjust_line
        ld      a,[val_FF11]
        and     $01
        call    show_it

        ld      a,[val_FF11]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF11]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF11]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF11]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF11]
        and     $20
        swap    a
        srl     a
        call    show_it

after_05
        ld      a,[cursor]
        cp      $04
        jr      nz,no_cursor_04

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_04

        call    adjust_line
        REPT    6
        dec     hl
        ENDR
        ld      a,$20
        REPT    2
        ld      [hl-],a
        ENDR

        ld      hl,text_04
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_04

no_cursor_04
        ld      a,[val_FF11]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF11]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_04
        ret

show_NR12
        ld      a,[cursor]
        cp      $08
        jr      nz,no_cursor_08

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_08

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_08
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_08

no_cursor_08
        call    adjust_line
        ld      a,[val_FF12]
        and     $01
        call    show_it

        ld      a,[val_FF12]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF12]
        and     $04
        srl     a
        srl     a
        call    show_it

after_08
        ld      a,[cursor]
        cp      $07
        jr      nz,no_cursor_07

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_07

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_07
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_07

no_cursor_07
        ld      a,[val_FF12]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_07
        ld      a,[cursor]
        cp      $06
        jr      nz,no_cursor_06

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_06

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_06
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_06

no_cursor_06
        ld      a,[val_FF12]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF12]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF12]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF12]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_06
        ret

show_NR13
        ld      a,[cursor]
        cp      $09
        jr      nz,no_cursor_09

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_09

        call    adjust_line
        ld      a,$20
        REPT    8
        ld      [hl-],a
        ENDR

        ld      hl,text_09
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_09

no_cursor_09
        call    adjust_line
        ld      a,[val_FF13]
        and     $01
        call    show_it

        ld      a,[val_FF13]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF13]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF13]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF13]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF13]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF13]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF13]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_09
        ret

show_NR14
        ld      a,[cursor]
        cp      $0D
        jr      nz,no_cursor_0D

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_0D

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_0D
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a

        jr      after_0D

no_cursor_0D
        call    adjust_line
        ld      a,[val_FF14]
        and     $01
        call    show_it

        ld      a,[val_FF14]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF14]
        and     $04
        srl     a
        srl     a
        call    show_it

after_0D
        ld      a,[cursor]
        cp      $0C
        jr      nz,no_cursor_0C

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_0C

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_1x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a

        jr      after_0C

no_cursor_0C
        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,[val_FF14]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF14]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF14]
        and     $20
        swap    a
        srl     a
        call    show_it

after_0C
        ld      a,[cursor]
        cp      $0B
        jr      nz,no_cursor_0B

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_0B

        call    adjust_line
        REPT    6
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_0B
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_0B

no_cursor_0B
        ld      a,[val_FF14]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_0B
        ld      a,[cursor]
        cp      $0A
        jr      nz,no_cursor_0A

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_0A

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_0A
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_0A

no_cursor_0A
        ld      a,[val_FF14]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_0A
        ret

show_NR21
        ld      a,[cursor]
        cp      $0F
        jr      nz,no_cursor_0F

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_0F

        call    adjust_line
        ld      a,$20
        REPT    6
        ld      [hl-],a
        ENDR

        ld      hl,text_0F
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_0F

no_cursor_0F
        call    adjust_line
        ld      a,[val_FF16]
        and     $01
        call    show_it

        ld      a,[val_FF16]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF16]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF16]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF16]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF16]
        and     $20
        swap    a
        srl     a
        call    show_it

after_0F
        ld      a,[cursor]
        cp      $0E
        jr      nz,no_cursor_0E

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_0E

        call    adjust_line
        REPT    6
        dec     hl
        ENDR
        ld      a,$20
        REPT    2
        ld      [hl-],a
        ENDR

        ld      hl,text_0E
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_0E

no_cursor_0E
        ld      a,[val_FF16]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF16]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_0E
        ret

show_NR22
        ld      a,[cursor]
        cp      $12
        jr      nz,no_cursor_12

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_12

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_12
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_12

no_cursor_12
        call    adjust_line
        ld      a,[val_FF17]
        and     $01
        call    show_it

        ld      a,[val_FF17]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF17]
        and     $04
        srl     a
        srl     a
        call    show_it

after_12
        ld      a,[cursor]
        cp      $11
        jr      nz,no_cursor_11

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_11

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_11
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_11

no_cursor_11
        ld      a,[val_FF17]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_11
        ld      a,[cursor]
        cp      $10
        jr      nz,no_cursor_10

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_10

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_10
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_10

no_cursor_10
        ld      a,[val_FF17]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF17]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF17]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF17]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_10
        ret

show_NR23
        ld      a,[cursor]
        cp      $13
        jr      nz,no_cursor_13

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_13

        call    adjust_line
        ld      a,$20
        REPT    8
        ld      [hl-],a
        ENDR

        ld      hl,text_13
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_13

no_cursor_13
        call    adjust_line
        ld      a,[val_FF18]
        and     $01
        call    show_it

        ld      a,[val_FF18]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF18]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF18]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF18]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF18]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF18]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF18]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_13
        ret

show_NR24
        ld      a,[cursor]
        cp      $17
        jr      nz,no_cursor_17

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_17

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_17
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_17

no_cursor_17
        call    adjust_line
        ld      a,[val_FF19]
        and     $01
        call    show_it

        ld      a,[val_FF19]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF19]
        and     $04
        srl     a
        srl     a
        call    show_it

after_17
        ld      a,[cursor]
        cp      $16
        jr      nz,no_cursor_16

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_16

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_2x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_16

no_cursor_16
        call    adjust_line
        REPT    3
        dec     hl
        ENDR

        ld      a,[val_FF19]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF19]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF19]
        and     $20
        swap    a
        srl     a
        call    show_it

after_16
        ld      a,[cursor]
        cp      $15
        jr      nz,no_cursor_15

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_15

        call    adjust_line
        REPT    6
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_15
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_15

no_cursor_15
        ld      a,[val_FF19]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_15
        ld      a,[cursor]
        cp      $14
        jr      nz,no_cursor_14

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_14

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_14
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_14

no_cursor_14
        ld      a,[val_FF19]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_14
        ret

show_NR30
        ld      a,[cursor]
        cp      $19
        jr      nz,no_cursor_19

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_19

        call    adjust_line
        ld      a,$20
        REPT    7
        ld      [hl-],a
        ENDR

        ld      hl,text_3x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_19

no_cursor_19
        call    adjust_line
        ld      a,[val_FF1A]
        and     $01
        call    show_it

        ld      a,[val_FF1A]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF1A]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1A]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1A]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF1A]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF1A]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_19
        ld      a,[cursor]
        cp      $18
        jr      nz,no_cursor_18

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_18

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_18
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_18

no_cursor_18
        ld      a,[val_FF1A]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_18
        ret

show_NR31
        ld      a,[cursor]
        cp      $1A
        jr      nz,no_cursor_1A

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_1A

        call    adjust_line
        ld      a,$20
        REPT    8
        ld      [hl-],a
        ENDR

        ld      hl,text_1A
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_1A

no_cursor_1A
        call    adjust_line
        ld      a,[val_FF1B]
        and     $01
        call    show_it

        ld      a,[val_FF1B]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF1B]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1B]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1B]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF1B]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF1B]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1B]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_1A
        ret

show_NR32
        ld      a,[cursor]
        cp      $1D
        jr      nz,no_cursor_1D

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_1D

        call    adjust_line
        ld      a,$20

        REPT    5
        ld      [hl-],a
        ENDR

        ld      hl,text_3x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_1D

no_cursor_1D
        call    adjust_line
        ld      a,[val_FF1C]
        and     $01
        call    show_it

        ld      a,[val_FF1C]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF1C]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1C]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1C]
        and     $10
        swap    a
        call    show_it

after_1D
        ld      a,[cursor]
        cp      $1C
        jr      nz,no_cursor_1C

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_1C

        call    adjust_line
        REPT    5
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a
        ld      [hl-],a

        ld      hl,text_1C
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_1C

no_cursor_1C
        ld      a,[val_FF1C]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF1C]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_1C
        ld      a,[cursor]
        cp      $1B
        jr      nz,no_cursor_1B

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_1B

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_3x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_1B

no_cursor_1B
        ld      a,[val_FF1C]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_1B
        ret

show_NR33
        ld      a,[cursor]
        cp      $1E
        jr      nz,no_cursor_1E

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_1E

        call    adjust_line
        ld      a,$20
        REPT    8
        ld      [hl-],a
        ENDR

        ld      hl,text_1E
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_1E

no_cursor_1E
        call    adjust_line
        ld      a,[val_FF1D]
        and     $01
        call    show_it

        ld      a,[val_FF1D]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF1D]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1D]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1D]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF1D]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF1D]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1D]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_1E
        ret

show_NR34
        ld      a,[cursor]
        cp      $22
        jr      nz,no_cursor_22

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_22

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_22
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_22

no_cursor_22
        call    adjust_line
        ld      a,[val_FF1E]
        and     $01
        call    show_it

        ld      a,[val_FF1E]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF1E]
        and     $04
        srl     a
        srl     a
        call    show_it

after_22
        ld      a,[cursor]
        cp      $21
        jr      nz,no_cursor_21

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_21

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_3x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_21

no_cursor_21
        ld      a,[val_FF1E]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF1E]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF1E]
        and     $20
        swap    a
        srl     a
        call    show_it

after_21
        ld      a,[cursor]
        cp      $20
        jr      nz,no_cursor_20

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_20

        call    adjust_line
        REPT    6
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_20
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_20

no_cursor_20
        ld      a,[val_FF1E]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_20
        ld      a,[cursor]
        cp      $1F
        jr      nz,no_cursor_1F

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_1F

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_1F
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_1F

no_cursor_1F
        ld      a,[val_FF1E]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_1F
        ret

show_NR41
        ld      a,[cursor]
        cp      $24
        jr      nz,no_cursor_24

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_24

        call    adjust_line
        ld      a,$20
        REPT    6
        ld      [hl-],a
        ENDR

        ld      hl,text_24
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_24

no_cursor_24
        call    adjust_line
        ld      a,[val_FF20]
        and     $01
        call    show_it

        ld      a,[val_FF20]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF20]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF20]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF20]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF20]
        and     $20
        swap    a
        srl     a
        call    show_it

after_24
        ld      a,[cursor]
        cp      $23
        jr      nz,no_cursor_23

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_23

        call    adjust_line
        ld      a,$20
        REPT    6
        dec     hl
        ENDR
        REPT    2
        ld      [hl-],a
        ENDR

        ld      hl,text_4x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_23

no_cursor_23
        ld      a,[val_FF20]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF20]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_23
        ret

show_NR42
        ld      a,[cursor]
        cp      $27
        jr      nz,no_cursor_27

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_27

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_27
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_27

no_cursor_27
        call    adjust_line
        ld      a,[val_FF21]
        and     $01
        call    show_it

        ld      a,[val_FF21]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF21]
        and     $04
        srl     a
        srl     a
        call    show_it

after_27
        ld      a,[cursor]
        cp      $26
        jr      nz,no_cursor_26

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_26

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_26
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_26

no_cursor_26
        ld      a,[val_FF21]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_26
        ld      a,[cursor]
        cp      $25
        jr      nz,no_cursor_25

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_25

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_25
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_25

no_cursor_25
        ld      a,[val_FF21]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF21]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF21]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF21]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_25
        ret

show_NR43
        ld      a,[cursor]
        cp      $2A
        jr      nz,no_cursor_2A

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_2A

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_2A
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_2A

no_cursor_2A
        call    adjust_line
        ld      a,[val_FF22]
        and     $01
        call    show_it

        ld      a,[val_FF22]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF22]
        and     $04
        srl     a
        srl     a
        call    show_it

after_2A
        ld      a,[cursor]
        cp      $29
        jr      nz,no_cursor_29

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_29

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_29
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_29

no_cursor_29
        ld      a,[val_FF22]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_29
        ld      a,[cursor]
        cp      $28
        jr      nz,no_cursor_28

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_28

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_28
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_28

no_cursor_28
        ld      a,[val_FF22]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF22]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF22]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF22]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_28
        ret

show_NR44
        ld      a,[cursor]
        cp      $2D
        jr      nz,no_cursor_2D

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_2D

        call    adjust_line
        ld      a,$20
        REPT    6
        ld      [hl-],a
        ENDR

        ld      hl,text_4x
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_2D

no_cursor_2D
        call    adjust_line
        ld      a,[val_FF23]
        and     $01
        call    show_it

        ld      a,[val_FF23]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF23]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF23]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF23]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF23]
        and     $20
        swap    a
        srl     a
        call    show_it

after_2D
        ld      a,[cursor]
        cp      $2C
        jr      nz,no_cursor_2C

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_2C

        call    adjust_line
        REPT    6
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_2C
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_2C

no_cursor_2C
        ld      a,[val_FF23]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_2C
        ld      a,[cursor]
        cp      $2B
        jr      nz,no_cursor_2B

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_2B

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_2B
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_2B

no_cursor_2B
        ld      a,[val_FF23]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_2B
        ret

show_NR50
        ld      a,[cursor]
        cp      $31
        jr      nz,no_cursor_31

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_31

        call    adjust_line
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_31
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_31

no_cursor_31
        call    adjust_line
        ld      a,[val_FF24]
        and     $01
        call    show_it

        ld      a,[val_FF24]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF24]
        and     $04
        srl     a
        srl     a
        call    show_it

after_31
        ld      a,[cursor]
        cp      $30
        jr      nz,no_cursor_30

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_30

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_30
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_30

no_cursor_30
        ld      a,[val_FF24]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_30
        ld      a,[cursor]
        cp      $2F
        jr      nz,no_cursor_2F

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_2F

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    3
        ld      [hl-],a
        ENDR

        ld      hl,text_2F
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_2F

no_cursor_2F
        ld      a,[val_FF24]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF24]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF24]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_2F
        ld      a,[cursor]
        cp      $2E
        jr      nz,no_cursor_2E

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_2E

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_2E
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_2E

no_cursor_2E
        ld      a,[val_FF24]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_2E
        ret

show_NR51
        ld      a,[cursor]
        cp      $39
        jr      nz,no_cursor_39

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_39

        call    adjust_line
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_39
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_39

no_cursor_39
        call    adjust_line
        ld      a,[val_FF25]
        and     $01
        call    show_it

after_39
        ld      a,[cursor]
        cp      $38
        jr      nz,no_cursor_38

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_38

        call    adjust_line
        dec     hl
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_38
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_38

no_cursor_38
        ld      a,[val_FF25]
        and     $02
        srl     a
        call    show_it

after_38
        ld      a,[cursor]
        cp      $37
        jr      nz,no_cursor_37

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_37

        call    adjust_line
        REPT    2
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_37
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_37

no_cursor_37
        ld      a,[val_FF25]
        and     $04
        srl     a
        srl     a
        call    show_it

after_37
        ld      a,[cursor]
        cp      $36
        jr      nz,no_cursor_36

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_36

        call    adjust_line
        REPT    3
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_36
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_36

no_cursor_36
        ld      a,[val_FF25]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_36
        ld      a,[cursor]
        cp      $35
        jr      nz,no_cursor_35

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_35

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_35
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_35

no_cursor_35
        ld      a,[val_FF25]
        and     $10
        swap    a
        call    show_it

after_35
        ld      a,[cursor]
        cp      $34
        jr      nz,no_cursor_34

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_34

        call    adjust_line
        REPT    5
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_34
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_34

no_cursor_34
        ld      a,[val_FF25]
        and     $20
        swap    a
        srl     a
        call    show_it

after_34
        ld      a,[cursor]
        cp      $33
        jr      nz,no_cursor_33

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_33

        call    adjust_line
        REPT    6
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_33
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_33

no_cursor_33
        ld      a,[val_FF25]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_33
        ld      a,[cursor]
        cp      $32
        jr      nz,no_cursor_32

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_32

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_32
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_32

no_cursor_32
        ld      a,[val_FF25]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_32
        ret

show_NR52
        ld      a,[cursor]
        cp      $3B
        jr      nz,no_cursor_3B

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_3B

        call    adjust_line
        ld      a,$20
        REPT    7
        ld      [hl-],a
        ENDR

        ld      hl,text_xx
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_3B

no_cursor_3B
        call    adjust_line
        ld      a,[val_FF26]
        and     $01
        call    show_it

        ld      a,[val_FF26]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF26]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF26]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF26]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF26]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF26]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

after_3B
        ld      a,[cursor]
        cp      $3A
        jr      nz,no_cursor_3A

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_3A

        call    adjust_line
        REPT    7
        dec     hl
        ENDR
        ld      a,$20
        ld      [hl-],a

        ld      hl,text_3A
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_3A

no_cursor_3A
        ld      a,[val_FF26]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_3A
        ret

show_0001
        ld      a,[cursor]
        cp      $3D
        jr      nz,no_cursor_3D

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_3D

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_3D
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_3D

no_cursor_3D
        call    adjust_line
        ld      a,[val_FF30]
        and     $01
        call    show_it

        ld      a,[val_FF30]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF30]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF30]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_3D
        ld      a,[cursor]
        cp      $3C
        jr      nz,no_cursor_3C

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_3C

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_3C
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_3C

no_cursor_3C
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF30]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF30]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF30]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF30]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_3C
        ret

show_0203
        ld      a,[cursor]
        cp      $3F
        jr      nz,no_cursor_3F

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_3F

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_3F
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_3F

no_cursor_3F
        call    adjust_line
        ld      a,[val_FF31]
        and     $01
        call    show_it

        ld      a,[val_FF31]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF31]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF31]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_3F
        ld      a,[cursor]
        cp      $3E
        jr      nz,no_cursor_3E

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_3E

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_3E
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_3E

no_cursor_3E
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF31]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF31]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF31]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF31]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_3E
        ret

show_0405
        ld      a,[cursor]
        cp      $41
        jr      nz,no_cursor_41

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_41

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_41
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_41

no_cursor_41
        call    adjust_line
        ld      a,[val_FF32]
        and     $01
        call    show_it

        ld      a,[val_FF32]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF32]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF32]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_41
        ld      a,[cursor]
        cp      $40
        jr      nz,no_cursor_40

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_40

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_40
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_40

no_cursor_40
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF32]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF32]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF32]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF32]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_40
        ret

show_0607
        ld      a,[cursor]
        cp      $43
        jr      nz,no_cursor_43

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_43

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_43
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_43

no_cursor_43
        call    adjust_line
        ld      a,[val_FF33]
        and     $01
        call    show_it

        ld      a,[val_FF33]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF33]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF33]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_43
        ld      a,[cursor]
        cp      $42
        jr      nz,no_cursor_42

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_42

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_42
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_42

no_cursor_42
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF33]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF33]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF33]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF33]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_42
        ret

show_0809
        ld      a,[cursor]
        cp      $45
        jr      nz,no_cursor_45

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_45

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_45
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_45

no_cursor_45
        call    adjust_line
        ld      a,[val_FF34]
        and     $01
        call    show_it

        ld      a,[val_FF34]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF34]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF34]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_45
        ld      a,[cursor]
        cp      $44
        jr      nz,no_cursor_44

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_44

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_44
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_44

no_cursor_44
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF34]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF34]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF34]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF34]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_44
        ret

show_0A0B
        ld      a,[cursor]
        cp      $47
        jr      nz,no_cursor_47

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_47

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_47
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_47

no_cursor_47
        call    adjust_line
        ld      a,[val_FF35]
        and     $01
        call    show_it

        ld      a,[val_FF35]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF35]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF35]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_47
        ld      a,[cursor]
        cp      $46
        jr      nz,no_cursor_46

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_46

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_46
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_46

no_cursor_46
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF35]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF35]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF35]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF35]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_46
        ret

show_0C0D
        ld      a,[cursor]
        cp      $49
        jr      nz,no_cursor_49

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_49

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_49
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_49

no_cursor_49
        call    adjust_line
        ld      a,[val_FF36]
        and     $01
        call    show_it

        ld      a,[val_FF36]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF36]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF36]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_49
        ld      a,[cursor]
        cp      $48
        jr      nz,no_cursor_48

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_48

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_48
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_48

no_cursor_48
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF36]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF36]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF36]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF36]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_48
        ret

show_0E0F
        ld      a,[cursor]
        cp      $4B
        jr      nz,no_cursor_4B

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_4B

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_4B
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_4B

no_cursor_4B
        call    adjust_line
        ld      a,[val_FF37]
        and     $01
        call    show_it

        ld      a,[val_FF37]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF37]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF37]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_4B
        ld      a,[cursor]
        cp      $4A
        jr      nz,no_cursor_4A

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_4A

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_4A
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_4A

no_cursor_4A
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF37]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF37]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF37]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF37]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_4A
        ret

show_1011
        ld      a,[cursor]
        cp      $4D
        jr      nz,no_cursor_4D

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_4D

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_4D
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_4D

no_cursor_4D
        call    adjust_line
        ld      a,[val_FF38]
        and     $01
        call    show_it

        ld      a,[val_FF38]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF38]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF38]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_4D
        ld      a,[cursor]
        cp      $4C
        jr      nz,no_cursor_4C

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_4C

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_4C
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_4C

no_cursor_4C
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF38]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF38]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF38]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF38]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_4C
        ret

show_1213
        ld      a,[cursor]
        cp      $4F
        jr      nz,no_cursor_4F

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_4F

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_4F
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_4F

no_cursor_4F
        call    adjust_line
        ld      a,[val_FF39]
        and     $01
        call    show_it

        ld      a,[val_FF39]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF39]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF39]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_4F
        ld      a,[cursor]
        cp      $4E
        jr      nz,no_cursor_4E

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_4E

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_4E
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_4E

no_cursor_4E
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF39]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF39]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF39]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF39]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_4E
        ret

show_1415
        ld      a,[cursor]
        cp      $51
        jr      nz,no_cursor_51

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_51

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_51
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_51

no_cursor_51
        call    adjust_line
        ld      a,[val_FF3A]
        and     $01
        call    show_it

        ld      a,[val_FF3A]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF3A]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3A]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_51
        ld      a,[cursor]
        cp      $50
        jr      nz,no_cursor_50

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_50

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_50
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_50

no_cursor_50
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF3A]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF3A]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF3A]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3A]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_50
        ret

show_1617
        ld      a,[cursor]
        cp      $53
        jr      nz,no_cursor_53

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_53

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_53
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_53

no_cursor_53
        call    adjust_line
        ld      a,[val_FF3B]
        and     $01
        call    show_it

        ld      a,[val_FF3B]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF3B]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3B]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_53
        ld      a,[cursor]
        cp      $52
        jr      nz,no_cursor_52

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_52

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_52
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_52

no_cursor_52
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF3B]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF3B]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF3B]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3B]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_52
        ret

show_1819
        ld      a,[cursor]
        cp      $55
        jr      nz,no_cursor_55

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_55

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_55
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_55

no_cursor_55
        call    adjust_line
        ld      a,[val_FF3C]
        and     $01
        call    show_it

        ld      a,[val_FF3C]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF3C]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3C]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_55
        ld      a,[cursor]
        cp      $54
        jr      nz,no_cursor_54

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_54

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_54
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_54

no_cursor_54
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF3C]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF3C]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF3C]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3C]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_54
        ret

show_1A1B
        ld      a,[cursor]
        cp      $57
        jr      nz,no_cursor_57

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_57

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_57
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_57

no_cursor_57
        call    adjust_line
        ld      a,[val_FF3D]
        and     $01
        call    show_it

        ld      a,[val_FF3D]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF3D]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3D]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_57
        ld      a,[cursor]
        cp      $56
        jr      nz,no_cursor_56

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_56

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_56
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_56

no_cursor_56
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF3D]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF3D]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF3D]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3D]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_56
        ret

show_1C1D
        ld      a,[cursor]
        cp      $59
        jr      nz,no_cursor_59

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_59

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_59
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_59

no_cursor_59
        call    adjust_line
        ld      a,[val_FF3E]
        and     $01
        call    show_it

        ld      a,[val_FF3E]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF3E]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3E]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_59
        ld      a,[cursor]
        cp      $58
        jr      nz,no_cursor_58

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_58

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_58
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_58

no_cursor_58
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF3E]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF3E]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF3E]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3E]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_58
        ret

show_1E1F
        ld      a,[cursor]
        cp      $5B
        jr      nz,no_cursor_5B

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_5B

        call    adjust_line
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_5B
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_5B

no_cursor_5B
        call    adjust_line
        ld      a,[val_FF3F]
        and     $01
        call    show_it

        ld      a,[val_FF3F]
        and     $02
        srl     a
        call    show_it

        ld      a,[val_FF3F]
        and     $04
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3F]
        and     $08
        srl     a
        srl     a
        srl     a
        call    show_it

after_5B
        ld      a,[cursor]
        cp      $5A
        jr      nz,no_cursor_5A

        ld      a,[blink]
        cp      $00
        jr      nz,no_cursor_5A

        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,$20
        REPT    4
        ld      [hl-],a
        ENDR

        ld      hl,text_5A
        ld      a,h
        ld      [TextH],a
        ld      a,l
        ld      [TextL],a
        jr      after_5A

no_cursor_5A
        call    adjust_line
        REPT    4
        dec     hl
        ENDR
        ld      a,[val_FF3F]
        and     $10
        swap    a
        call    show_it

        ld      a,[val_FF3F]
        and     $20
        swap    a
        srl     a
        call    show_it

        ld      a,[val_FF3F]
        and     $40
        swap    a
        srl     a
        srl     a
        call    show_it

        ld      a,[val_FF3F]
        and     $80
        swap    a
        srl     a
        srl     a
        srl     a
        call    show_it

after_5A
        ret

show_it
        cp      $0A
        jr      nc,alpha
        add     $30
        jr      go_show

alpha
        add     $37

go_show
        ld      [hl-],a
        ret

update_NR
        ld      a,[val_FF10]
        ldh     [$10],a         ;set NR 10
        ld      a,[val_FF11]
        ldh     [$11],a         ;set NR 11
        ld      a,[val_FF12]
        ldh     [$12],a         ;set NR 12
        ld      a,[val_FF13]
        ldh     [$13],a         ;set NR 13
        ld      a,[val_FF14]
        ldh     [$14],a         ;set NR 14

        ld      a,[val_FF16]
        ldh     [$16],a         ;set NR 21
        ld      a,[val_FF17]
        ldh     [$17],a         ;set NR 22
        ld      a,[val_FF18]
        ldh     [$18],a         ;set NR 23
        ld      a,[val_FF19]
        ldh     [$19],a         ;set NR 24

        ld      a,[val_FF1A]
        ldh     [$1A],a         ;set NR 30
        ld      a,[val_FF1B]
        ldh     [$1B],a         ;set NR 31
        ld      a,[val_FF1C]
        ldh     [$1C],a         ;set NR 32
        ld      a,[val_FF1D]
        ldh     [$1D],a         ;set NR 33
        ld      a,[val_FF1E]
        ldh     [$1E],a         ;set NR 34

        ld      a,[val_FF20]
        ldh     [$20],a         ;set NR 41
        ld      a,[val_FF21]
        ldh     [$21],a         ;set NR 42
        ld      a,[val_FF22]
        ldh     [$22],a         ;set NR 43
        ld      a,[val_FF23]
        ldh     [$23],a         ;set NR 44

        ld      a,[val_FF24]
        ldh     [$24],a         ;set NR 50
        ld      a,[val_FF25]
        ldh     [$25],a         ;set NR 51
        ld      a,[val_FF26]
        ldh     [$26],a         ;set NR 52

        ld      a,[val_FF30]
        ldh     [$30],a         ;set WaveRAM
        ld      a,[val_FF31]
        ldh     [$31],a         ;set WaveRAM
        ld      a,[val_FF32]
        ldh     [$32],a         ;set WaveRAM
        ld      a,[val_FF33]
        ldh     [$33],a         ;set WaveRAM
        ld      a,[val_FF34]
        ldh     [$34],a         ;set WaveRAM
        ld      a,[val_FF35]
        ldh     [$35],a         ;set WaveRAM
        ld      a,[val_FF36]
        ldh     [$36],a         ;set WaveRAM
        ld      a,[val_FF37]
        ldh     [$37],a         ;set WaveRAM
        ld      a,[val_FF38]
        ldh     [$38],a         ;set WaveRAM
        ld      a,[val_FF39]
        ldh     [$39],a         ;set WaveRAM
        ld      a,[val_FF3A]
        ldh     [$3A],a         ;set WaveRAM
        ld      a,[val_FF3B]
        ldh     [$3B],a         ;set WaveRAM
        ld      a,[val_FF3C]
        ldh     [$3C],a         ;set WaveRAM
        ld      a,[val_FF3D]
        ldh     [$3D],a         ;set WaveRAM
        ld      a,[val_FF3E]
        ldh     [$3E],a         ;set WaveRAM
        ld      a,[val_FF3F]
        ldh     [$3F],a         ;set WaveRAM

        ret

initialize

        di
        xor     a
        ldh     [$FF],a         ;disable interrupts

.wait                         
        ldh     a,[$44]         ;LY LCDC compare
        cp      144
        jr      nz,.wait
        xor     a
        ldh     [$40],a         ;LCDC lcd control

        ld      hl,ibm_characters       ;load IBMfont
        ld      de,$8000
        ld      bc,8*256
        call    mem_CopyMono

        ld      hl,custom_map
        ld      de,display_col  ;+0+(SCRN_VY_B*10)
        ld      bc,32*20
        call    mem_Copy        ;_offset_custom

        ld      a,$01
        ldh     [$42],a         ;SCY Scroll Y
        xor     a
        ldh     [$43],a         ;SCX Scroll X

        ld      a,$E4
        ldh     [$51],a         ;BGP

        ld      a,%10010001
        ldh     [$40],a

        xor     a

        ld      [FirstLine],a
        ld      [TextH],a
        ld      [TextL],a
        ld      [UP_debounce],a
        ld      [DOWN_debounce],a
        ld      [LEFT_debounce],a
        ld      [RIGHT_debounce],a
        ld      [SELECT_debounce],a
        ld      [START_debounce],a
        ld      [A_debounce],a
        ld      [B_debounce],a
        ld      [button_data],a
        ld      [button_data_chg],a
        ld      [button_debounced],a
        ld      [NR_update],a
        ld      [cursor],a
        ld      [blink],a
        ld      [timediv],a

        ld      a,$80
        ldh     [$26],a       ;all sound on

        ld      a,$FF
        ldh     [$25],a       ;output sound 1 to SO1 & SO2 terminals

        ldh     a,[$10]
        ld      [val_FF10],a
        ldh     a,[$11]
        ld      [val_FF11],a
        ldh     a,[$12]
        ld      [val_FF12],a
        ldh     a,[$13]
        ld      [val_FF13],a
        ldh     a,[$14]
        ld      [val_FF14],a
        ldh     a,[$16]
        ld      [val_FF16],a
        ldh     a,[$17]
        ld      [val_FF17],a
        ldh     a,[$18]
        ld      [val_FF18],a
        ldh     a,[$19]
        ld      [val_FF19],a
        ldh     a,[$1A]
        ld      [val_FF1A],a
        ldh     a,[$1B]
        ld      [val_FF1B],a
        ldh     a,[$1C]
        ld      [val_FF1C],a
        ldh     a,[$1D]
        ld      [val_FF1D],a
        ldh     a,[$1E]
        ld      [val_FF1E],a
        ldh     a,[$20]
        ld      [val_FF20],a
        ldh     a,[$21]
        ld      [val_FF21],a
        ldh     a,[$22]
        ld      [val_FF22],a
        ldh     a,[$23]
        ld      [val_FF23],a
        ldh     a,[$24]
        ld      [val_FF24],a
        ldh     a,[$25]
        ld      [val_FF25],a
        ldh     a,[$26]
        ld      [val_FF26],a

        ldh     a,[$30]
        ld      [val_FF30],a
        ldh     a,[$31]
        ld      [val_FF31],a
        ldh     a,[$32]
        ld      [val_FF32],a
        ldh     a,[$33]
        ld      [val_FF33],a
        ldh     a,[$34]
        ld      [val_FF34],a
        ldh     a,[$35]
        ld      [val_FF35],a
        ldh     a,[$36]
        ld      [val_FF36],a
        ldh     a,[$37]
        ld      [val_FF37],a
        ldh     a,[$38]
        ld      [val_FF38],a
        ldh     a,[$39]
        ld      [val_FF39],a
        ldh     a,[$3A]
        ld      [val_FF3A],a
        ldh     a,[$3B]
        ld      [val_FF3B],a
        ldh     a,[$3C]
        ld      [val_FF3C],a
        ldh     a,[$3D]
        ld      [val_FF3D],a
        ldh     a,[$3E]
        ld      [val_FF3E],a
        ldh     a,[$3F]
        ld      [val_FF3F],a

        call    update_NR

        ld      a,%00000000
        ldh     [$56],a

        ld      a,256-timer_div
        ldh     [$06],a

        ld      a,%00000100     ;start timer 4096 Hz
        ldh     [$07],a

        ld      a,%00000101
        ldh     [$FF],a         ;timer int

        ei

        ret
        
        SECTION "ConstStr",CODE

ibm_characters
        chr_IBMfont

        ;length  000000000111111111
        ;        123456789012345678
text_01 DB      "1)SwpTime 0=NoSwp "
text_02 DB      "1)0=SwpUp 1=SwpDn "
text_03 DB      "1)SwpShiftNumber  "
text_04 DB      "1)WavePatternDuty "
text_05 DB      "1)SoundLengthData "
text_06 DB      "1)IniEnvlVol 0=Off"
text_07 DB      "1)Envl 0=Att 1=Amp"
text_08 DB      "1)EnvlSwp 0=NoEnvl"
text_09 DB      "1)Freq Lower 8bits"
text_0A DB      "1)Initialize Sound"
text_0B DB      "1)0=Consctv 1=Cntr"
text_0D DB      "1)Freq Upper 3bits"
text_0E DB      "2)WavePatternDuty "
text_0F DB      "2)SoundLengthData "
text_10 DB      "2)IniEnvlVol 0=Off"
text_11 DB      "2)Envl 0=Att 1=Amp"
text_12 DB      "2)EnvlSwp 0=NoEnvl"
text_13 DB      "2)Freq Lower 8bits"
text_14 DB      "2)Initialize Sound"
text_15 DB      "2)0=Consctv 1=Cntr"
text_17 DB      "2)Freq Upper 3bits"
text_18 DB      "3)0=SndOff 1=SndOn"
text_1A DB      "3)Sound Length    "
text_1C DB      "3)WaveRAM OutLevel"
text_1E DB      "3)Freq Lower 8bits"
text_1F DB      "3)Initialize Sound"
text_20 DB      "3)0=Consctv 1=Cntr"
text_22 DB      "3)Freq Upper 3bits"
text_24 DB      "4)SoundLengthData "
text_25 DB      "4)IniEnvlVol 0=Off"
text_26 DB      "4)Envl 0=Att 1=Amp"
text_27 DB      "4)EnvlSwp 0=NoEnvl"
text_28 DB      "4)Shift Clock Freq"
text_29 DB      "4)CntStep 0=15 1=7"
text_2A DB      "4)Freq Div Ratio  "
text_2B DB      "4)Initialize Sound"
text_2C DB      "4)0=Consctv 1=Cntr"
text_2E DB      "Vin>SO2 0=Off 1=On"
text_2F DB      "SO2 Output Level  "
text_30 DB      "Vin>SO1 0=Off 1=On"
text_31 DB      "SO1 Output Level  "
text_32 DB      "Sound4 to SO2 term"
text_33 DB      "Sound3 to SO2 term"
text_34 DB      "Sound2 to SO2 term"
text_35 DB      "Sound1 to SO2 term"
text_36 DB      "Sound4 to SO1 term"
text_37 DB      "Sound3 to SO1 term"
text_38 DB      "Sound2 to SO1 term"
text_39 DB      "Sound1 to SO1 term"
text_3A DB      "0=AllOff 1=SoundOn"
text_3C DB      "WaveRAM Sample 00 "
text_3D DB      "WaveRAM Sample 01 "
text_3E DB      "WaveRAM Sample 02 "
text_3F DB      "WaveRAM Sample 03 "
text_40 DB      "WaveRAM Sample 04 "
text_41 DB      "WaveRAM Sample 05 "
text_42 DB      "WaveRAM Sample 06 "
text_43 DB      "WaveRAM Sample 07 "
text_44 DB      "WaveRAM Sample 08 "
text_45 DB      "WaveRAM Sample 09 "
text_46 DB      "WaveRAM Sample 0A "
text_47 DB      "WaveRAM Sample 0B "
text_48 DB      "WaveRAM Sample 0C "
text_49 DB      "WaveRAM Sample 0D "
text_4A DB      "WaveRAM Sample 0E "
text_4B DB      "WaveRAM Sample 0F "
text_4C DB      "WaveRAM Sample 10 "
text_4D DB      "WaveRAM Sample 11 "
text_4E DB      "WaveRAM Sample 12 "
text_4F DB      "WaveRAM Sample 13 "
text_50 DB      "WaveRAM Sample 14 "
text_51 DB      "WaveRAM Sample 15 "
text_52 DB      "WaveRAM Sample 16 "
text_53 DB      "WaveRAM Sample 17 "
text_54 DB      "WaveRAM Sample 18 "
text_55 DB      "WaveRAM Sample 19 "
text_56 DB      "WaveRAM Sample 1A "
text_57 DB      "WaveRAM Sample 1B "
text_58 DB      "WaveRAM Sample 1C "
text_59 DB      "WaveRAM Sample 1D "
text_5A DB      "WaveRAM Sample 1E "
text_5B DB      "WaveRAM Sample 1F "
text_1x DB      "1)Undefined bit(s)"
text_2x DB      "2)Undefined bit(s)"
text_3x DB      "3)Undefined bit(s)"
text_4x DB      "4)Undefined bit(s)"
text_xx DB      "Undefined bit(s)  "
     
text_FirstLine
        DB      "1011121314161718191A1B1C1D1E20212223"
text_NextLine
        DB      "242526303132333435363738393A3B3C3D3E3F"

display_table
        DW      pos_0
        DW      pos_1
        DW      pos_2
        DW      pos_3
        DW      pos_4
        DW      pos_5
        DW      pos_6

pos_table
        DW      show_pos0
        DW      show_pos1
        DW      show_pos2
        DW      show_pos3
        DW      show_pos4
        DW      show_pos5
        DW      show_pos6
        DW      show_text

show_table
        DW      show_NR10
        DW      show_NR11
        DW      show_NR12
        DW      show_NR13
        DW      show_NR14
        DW      show_NR21
        DW      show_NR22
        DW      show_NR23
        DW      show_NR24
        DW      show_NR30
        DW      show_NR31
        DW      show_NR32
        DW      show_NR33
        DW      show_NR34
        DW      show_NR41
        DW      show_NR42
        DW      show_NR43
        DW      show_NR44
        DW      show_NR50
        DW      show_NR51
        DW      show_NR52
        DW      show_0001
        DW      show_0203
        DW      show_0405
        DW      show_0607
        DW      show_0809
        DW      show_0A0B
        DW      show_0C0D
        DW      show_0E0F
        DW      show_1011
        DW      show_1213
        DW      show_1415
        DW      show_1617
        DW      show_1819
        DW      show_1A1B
        DW      show_1C1D
        DW      show_1E1F
 
        SECTION "GB_ram",BSS

FirstLine               DS      1
TextH                   DS      1
TextL                   DS      1

UP_debounce             DS      1
DOWN_debounce           DS      1
LEFT_debounce           DS      1
RIGHT_debounce          DS      1
SELECT_debounce         DS      1
START_debounce          DS      1
A_debounce              DS      1
B_debounce              DS      1

button_data             DS      1
button_data_chg         DS      1
button_debounced        DS      1

NR_update               DS      1

cursor                  DS      1

blink                   DS      1
timediv                 DS      1

val_FF10                DS      1
val_FF11                DS      1
val_FF12                DS      1
val_FF13                DS      1
val_FF14                DS      1
val_FF16                DS      1
val_FF17                DS      1
val_FF18                DS      1
val_FF19                DS      1
val_FF1A                DS      1
val_FF1B                DS      1
val_FF1C                DS      1
val_FF1D                DS      1
val_FF1E                DS      1
val_FF20                DS      1
val_FF21                DS      1
val_FF22                DS      1
val_FF23                DS      1
val_FF24                DS      1
val_FF25                DS      1
val_FF26                DS      1
val_FF30                DS      1
val_FF31                DS      1
val_FF32                DS      1
val_FF33                DS      1
val_FF34                DS      1
val_FF35                DS      1
val_FF36                DS      1
val_FF37                DS      1
val_FF38                DS      1
val_FF39                DS      1
val_FF3A                DS      1
val_FF3B                DS      1
val_FF3C                DS      1
val_FF3D                DS      1
val_FF3E                DS      1
val_FF3F                DS      1

custom_mapWidth  EQU 32
custom_mapHeight EQU 32
custom_mapBank   EQU 0

 SECTION "map", HOME

custom_map::
;   00000000011111111112222222222333
;   12345678901234567890123456789012
DB "ͻ            "   ;NOTE : the last 12 chars (blanks) 
DB "GBSoundXperimenter            "   ;       are needed, but they DO NOT
DB "v1.3 by MJCulross             "   ;       show up on the display screen
DB "͹            "
DB "FF..: ........ $..            "
DB "FF..: ........ $..            "
DB "FF..: ........ $..            "
DB "FF..: ........ $..            "
DB "FF..: ........ $..            "
DB "FF..: ........ $..            "
DB "FF..: ........ $..            "
DB "͹            "
DB "                              "
DB "͹            "
DB "(A)Play (B)PlayAll            "
DB "  Modify Value              "
DB " .. Move Cursor               "
DB "ͼ            "
DB "                                "   ;(empty)
DB "                                "   ;(empty)
